!```````````````````````````````````````````````````````````````````
module manage_NetCDF_classic_mod

!+ ("mNc" for short); Module that provides a slightly easier interface/API with
!   which to work with NetCDF-format data files that *either* use the "classic"
!   header *or* use only the features found in the "classic" header (no advanced
!   HDF5 features).  This includes all NetCDF-3 files and many NetCDF-4 files.
!
!------------ EXAMPLE USAGE -------------
!
!   use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
!   use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj,  &
!        mNc_populate_f_md_obj_r, mNc_recycle_f_md_obj, mNc_check,  &
!        mNc_populate_f_md_obj_w, mNc_MAX_NAME_LEN, mNc_CHAR_type,  &
!        mNc_FLOAT_type, mNc_file_md_Att_obj, mNc_define_f_md_Att_obj
!   use manage_NetCDF_classic_mod_rv, only: mNc_read_var
!   use manage_NetCDF_classic_mod_wv, only: mNc_write_var
!
!   integer :: n_fn
!   character(len=MAX_FILE_PATH_LEN), allocatable, dimension(:) ::  &
!        fn_pfx  ! {n_fn}
!
!   character(len=MAX_FILE_PATH_LEN) :: str_tmp
!
!   integer :: i_fn, i, ic, ia
!
!   type(mNc_file_metadata_obj), allocatable, dimension(:) :: file_in
!   type(mNc_file_metadata_obj), allocatable, dimension(:) :: file_out
!
!   real(f4), pointer, dimension(:,:) :: u_avg_ptr => NULL()
!
!   integer :: n_dims, n_vars, n_g_Atts, max_var_ndims, max_var_nAtts
!   character(len=mNc_MAX_NAME_LEN), allocatable, dimension(:) :: dim_name,  &
!        var_name
!   integer, allocatable, dimension(:) :: dimv, var_type, var_ndims, var_nAtts
!   integer, allocatable, dimension(:,:) :: var_dimids
!   type(mNc_file_md_Att_obj), allocatable, dimension(:) :: g_Att
!   type(mNc_file_md_Att_obj), allocatable, dimension(:,:) :: var_Att
!   character(len=mNc_MAX_NAME_LEN) :: tmp_str
!
!   ! Populate the NetCDF file object(s) with metadata:
!
!   n_fn = 2
!   allocate(fn_pfx(n_fn), file_in(n_fn), file_out(n_fn))
!
!   fn_pfx(1) = blah1.nc
!   fn_pfx(2) = blah2.nc
!   do i_fn=1,n_fn
!      call mNc_populate_f_md_obj_r(fn_pfx(i_fn), file_in(i_fn))
!   end do
!
!   !--- Read in and process data fields:
!
!      ! Snapshot #9 in file; afterwards, u_avg_ptr holds the field data: 
!   i_fn = 1
!   call mNc_read_var(file_in(i_fn), 'u_avg', u_avg_ptr,  &
!        opt_start=(/ 1, 1, 1, 9 /), opt_count=(/ -1, -1, -1, 1 /))
!, opt_stride)
!
!   do i_fn=1,n_fn
!      call mNc_recycle_f_md_obj(file_in(i_fn)) ! Close file, free object memory
!   end do
!
!   !---- Write output file(s):
!
!   i_fn = 1
!   n_dims = 2; n_vars = 3; n_g_Atts = 2
!
!   allocate(dim_name(n_dims), dimv(n_dims))
!   i = 1; dim_name(i) = 'longitude'; dimv(i) = 240
!   i = 2; dim_name(i) = 'latitude'; dimv(i) = 240 
!
!   allocate(var_name(n_vars), var_type(n_vars), var_ndims(n_vars),  &
!        var_nAtts(n_vars))
!
!   allocate(g_Att(n_g_Atts))
!
!   i = 1; tmp_str = 'guinea pigs'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(g_Att(i), 'blah', i, ic, tmp_str)
!   i = 2; call mNc_define_f_md_Att_obj(g_Att(i), 'Grape', i, 2, (/ 3., 6. /))
!
!   max_var_ndims = 2; max_var_nAtts = 2
!   allocate(var_dimids(max_var_ndims, n_vars), var_Att(max_var_nAtts, n_vars))
!
!   i = 1; var_name(i) = 'latitude'; var_type(i) = mNc_FLOAT_type
!   var_ndims(i) = 1; var_nAtts(i) = 2
!   var_dimids(1:var_ndims(i),i) = (/ 2 /)
!   ia = 1; tmp_str = 'latitude'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_str)
!   ia = 2; tmp_str = 'degrees_N'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_str)
!
!   i = 2; var_name(i) = 'longitude'; var_type(i) = mNc_FLOAT_type
!   var_ndims(i) = 1; var_nAtts(i) = 2
!   var_dimids(1:var_ndims(i),i) = (/ 1 /)
!   ia = 1; tmp_str = 'longitude'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_str)
!   ia = 2; tmp_str = 'degrees_E'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_str)
!
!   i = 3; var_name(i) = 'u_avg'; var_type(i) = mNc_FLOAT_type
!   var_ndims(i) = 2; var_nAtts(i) = 2
!   var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
!   ia = 1; tmp_str = 'west-east wind component'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_str)
!   ia = 2; tmp_str = 'm/s'; ic = len_trim(tmp_str)
!   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_str)
!
!   str_tmp = trim(fn_pfx(i_fn))//'-out.nc'
!   call mNc_populate_f_md_obj_w(str_tmp, file_out(i_fn), n_dims, n_vars,  &
!        n_g_Atts, dim_name, dimv, g_Att, var_name, var_type, var_ndims,  &
!        var_nAtts, var_dimids, var_Att)
!   
!   call mNc_write_var(file_out(i_fn), 'u_avg', u_avg_ptr)
!             !, opt_start, opt_count, opt_stride, opt_var_idx)
!
!   ! Finalize file, free object memory:
!   call mNc_recycle_md_obj(file_out(i_fn))
!
!------------ END EXAMPLE USAGE -------------

use univ_kind_defs_mod, only: i1, i2, i4, f4, f8
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use netcdf, only: NF90_MAX_NAME, NF90_BYTE, NF90_CHAR, NF90_SHORT, NF90_INT,  &
     NF90_FLOAT, NF90_DOUBLE

implicit none

private

! Derived-type object to hold relevant NetCDF metadata "attributes":

type mNc_file_md_Att_obj

   private

   logical :: is_populated = .FALSE.

   character(len=NF90_MAX_NAME) :: name   ! Name
   integer :: id,  &   ! ID
              type,  &  ! Data type
              dimv   ! Dimension (length)

   integer(i1), pointer, dimension(:) :: i1_data => NULL()
   character(len=1), pointer, dimension(:) :: c_data => NULL()
   integer(i2), pointer, dimension(:) :: i2_data => NULL()
   integer(i4), pointer, dimension(:) :: i4_data => NULL()
   real(f4), pointer, dimension(:) :: f4_data => NULL()
   real(f8), pointer, dimension(:) :: f8_data => NULL()

end type mNc_file_md_Att_obj

! Derived-type object to hold relevant NetCDF file metadata: 

type mNc_file_metadata_obj

   private

   logical :: is_populated = .FALSE.

   character(len=MAX_FILE_PATH_LEN) ::  &
        file_name     ! Name of NetCDF file (with path, if desired)

   integer :: ncid     ! NetCDF file ID

   integer :: n_dims,  &     ! Number of defined dimensions in the file
              n_vars,  &       ! Number of defined variables in the file
              n_g_Atts       ! Number of defined global attributes in the file

   integer :: unlimited_dimid   ! Dimension ID of any unlimited dimension
   logical :: unlimited_dim_exists
   
   ! Dimension information {n_dims}:

   integer, pointer, dimension(:) :: dimid => NULL(),  &  ! ID
                                     dimv => NULL()    ! Length (value)
   character(len=NF90_MAX_NAME), pointer, dimension(:) ::  &
        dim_name => NULL()  ! Name

   ! Variable information {n_vars}:

   integer, pointer, dimension(:) :: varid => NULL(),  &  ! ID
                                     var_name_len => NULL(),  &  ! Name length
                                     var_type => NULL(),  &  ! Data type
                                     var_ndims => NULL(),  &  ! # of dims
                                     var_nAtts => NULL()  ! # of attributes

   character(len=NF90_MAX_NAME), pointer, dimension(:) ::  &
        var_name => NULL()  ! Name
   integer :: max_var_ndims, max_var_nAtts

   integer, pointer, dimension(:,:) ::  &
        var_dimids => NULL()  ! Dim IDs for each var {max_var_ndims, n_vars}


   ! Variable attribute information {max_var_nAtts, n_vars}:   

   type(mNc_file_md_Att_obj), pointer, dimension(:,:) :: var_Att => NULL()

   ! Global attribute information {n_g_Atts}:

   type(mNc_file_md_Att_obj), pointer, dimension(:) :: g_Att => NULL()

end type mNc_file_metadata_obj

integer, public, parameter :: mNc_MAX_CHAR_ATT_LEN = 131072
character(len=mNc_MAX_CHAR_ATT_LEN) :: char_att_str_tmp

! Provide selected "aliases" to NetCDF library parameters:

integer, public, parameter :: mNc_MAX_NAME_LEN = NF90_MAX_NAME

! NOTE: No native 8-byte integer support in NetCDF v3 and v4_classic
integer, public, parameter :: mNc_BYTE_type = NF90_BYTE,  &   ! 1-byte integer 
                              mNc_CHAR_type = NF90_CHAR,  &   ! 1 character
                              mNc_SHORT_type = NF90_SHORT,  & ! 2-byte integer
                              mNc_INT_type = NF90_INT,  &     ! 4-byte integer
                              mNc_FLOAT_type = NF90_FLOAT,  & ! 4-byte real
                              mNc_DOUBLE_type = NF90_DOUBLE   ! 8-byte real

! Flexible procedure interfaces:

interface mNc_define_f_md_Att_obj
   module procedure mNc_define_f_md_Att_obj_i1, mNc_define_f_md_Att_obj_c,  &
        mNc_define_f_md_Att_obj_i2, mNc_define_f_md_Att_obj_i4,  &
        mNc_define_f_md_Att_obj_f4, mNc_define_f_md_Att_obj_f8
end interface

interface mNc_extract_g_Att_val
   module procedure mNc_extract_g_Att_val_i1, mNc_extract_g_Att_val_c,  &
        mNc_extract_g_Att_val_i2, mNc_extract_g_Att_val_i4,  &
        mNc_extract_g_Att_val_f4, mNc_extract_g_Att_val_f8
end interface

interface mNc_extract_var_Att_val
   module procedure mNc_extract_var_Att_val_i1, mNc_extract_var_Att_val_c,  &
        mNc_extract_var_Att_val_i2, mNc_extract_var_Att_val_i4,  &
        mNc_extract_var_Att_val_f4, mNc_extract_var_Att_val_f8
end interface

interface mNc_var_ndims
   module procedure mNc_var_ndims_n
   module procedure mNc_var_ndims_i
end interface

interface mNc_var_dims
   module procedure mNc_var_dims_n
   module procedure mNc_var_dims_i
end interface

interface mNc_var_dimval
   module procedure mNc_var_dimval_n
   module procedure mNc_var_dimval_i
end interface

interface mNc_dim_val
   module procedure mNc_dim_val_n
   module procedure mNc_dim_val_i
end interface

public :: mNc_file_metadata_obj, mNc_file_md_Att_obj, mNc_var_idx,  &
     mNc_recycle_f_md_obj, mNc_populate_f_md_obj_w, mNc_populate_f_md_obj_r,  &
     mNc_var_ndims, mNc_var_dims, mNc_check, mNc_file_ID,  &
     mNc_define_f_md_Att_obj, mNc_recycle_f_md_Att_obj, mNc_sync_with_file,  &
     mNc_extract_g_Att_val, mNc_extract_var_Att_val, mNc_var_Att_exists,  &
     mNc_g_Att_exists, mNc_copy_f_md_Att_obj, mNc_dim_val, mNc_close,  &
     mNc_f_md_populated, mNc_f_md_var_type, mNc_f_md_Att_populated,  &
     mNc_dim_idx, mNc_var_dimval, mNc_is_the_unlimited_dim


contains


include 'manage_NetCDF_classic.F90.inc01'


!```````````````````````````````````````````````````````````````````
subroutine mNc_populate_f_md_obj_r(nc_file_name, f_md_obj, open_as_readwrite)

!+ Populate the elements of an object of type 'mNc_file_metadata_obj' (from
!   existing file data)

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use netcdf, only: nf90_open, NF90_NOWRITE, NF90_MAX_NAME, nf90_inquire,  &
     nf90_inquire_dimension, nf90_inquire_variable, nf90_inq_attname,  &
     nf90_inquire_attribute, nf90_get_att, NF90_GLOBAL, NF90_WRITE

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: nc_file_name
logical, optional, intent(IN) :: open_as_readwrite

type(mNc_file_metadata_obj), intent(INOUT) :: f_md_obj

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_populate_f_md_obj_r'

character(len=256) :: error_msg
integer :: ic, i, e, max_tmp1, max_tmp2, ia
logical :: there_are_g_Atts, open_as_rw

!=== Executable statements:

open_as_rw = .FALSE.
if (present(open_as_readwrite)) open_as_rw = open_as_readwrite

if (f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'This NetCDF file metadata object is already populated; use '// &
     '`mNc_recycle_md_obj` first')

!--- Open the existing NetCDF file:

ic = len_trim(nc_file_name)
f_md_obj%file_name = nc_file_name(1:ic)   ! Store filename string

if (ic > NF90_MAX_NAME) then   ! Check for NetCDF string length limit
   write (error_msg, '(a,i6,a)')  &
        'The given NetCDF file name is too long; must be <= ',  &
        NF90_MAX_NAME, ' characters in length'
   call exception(procedure_name, FATAL, error_msg)
end if

if (open_as_rw) then
   call mNc_check(nf90_open(nc_file_name(1:ic), NF90_WRITE, f_md_obj%ncid))
else
   call mNc_check(nf90_open(nc_file_name(1:ic), NF90_NOWRITE, f_md_obj%ncid))
end if

!--- Inquire about (and set) important file parameters; allocate relevant
!     object elements:

call mNc_check(nf90_inquire(f_md_obj%ncid, f_md_obj%n_dims, f_md_obj%n_vars,  &
     f_md_obj%n_g_Atts, f_md_obj%unlimited_dimid))

there_are_g_Atts = ( f_md_obj%n_g_Atts > 0 )
f_md_obj%unlimited_dim_exists = ( f_md_obj%unlimited_dimid /= -1 )

allocate(f_md_obj%dimid(f_md_obj%n_dims), f_md_obj%dim_name(f_md_obj%n_dims),  &
     f_md_obj%dimv(f_md_obj%n_dims), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'allocation #01')
allocate(f_md_obj%varid(f_md_obj%n_vars), f_md_obj%var_name(f_md_obj%n_vars),  &
     f_md_obj%var_type(f_md_obj%n_vars), f_md_obj%var_ndims(f_md_obj%n_vars),  &
     f_md_obj%var_nAtts(f_md_obj%n_vars),  &
     f_md_obj%var_name_len(f_md_obj%n_vars), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'allocation #02')
if (there_are_g_Atts) then
   allocate(f_md_obj%g_Att(f_md_obj%n_g_Atts), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #03')
end if

!--- Obtain dimension sizes and names:

do i=1,f_md_obj%n_dims
   f_md_obj%dimid(i) = i
   call mNc_check(nf90_inquire_dimension(f_md_obj%ncid, f_md_obj%dimid(i),  &
        f_md_obj%dim_name(i), f_md_obj%dimv(i)))
end do

!--- Obtain global attribute information (if any):

if (there_are_g_Atts) then
   do i=1,f_md_obj%n_g_Atts
      f_md_obj%g_Att(i)%id = i
      call mNc_check(nf90_inq_attname(f_md_obj%ncid, NF90_GLOBAL,  &
           f_md_obj%g_Att(i)%id, f_md_obj%g_Att(i)%name))
      call mNc_check(nf90_inquire_attribute(f_md_obj%ncid, NF90_GLOBAL,  &
           f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%type,  &
           f_md_obj%g_Att(i)%dimv))

      select case (f_md_obj%g_Att(i)%type)
      case (NF90_BYTE)
         allocate(f_md_obj%g_Att(i)%i1_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04a')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i1_data))
      case (NF90_CHAR)
         allocate(f_md_obj%g_Att(i)%c_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04aa')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, char_att_str_tmp))
         do ic=1,len_trim(char_att_str_tmp)
            f_md_obj%g_Att(i)%c_data(ic) = char_att_str_tmp(ic:ic)
         end do
      case (NF90_SHORT)
         allocate(f_md_obj%g_Att(i)%i2_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04b')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i2_data))
      case (NF90_INT)
         allocate(f_md_obj%g_Att(i)%i4_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04c')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i4_data))
      case (NF90_FLOAT)
         allocate(f_md_obj%g_Att(i)%f4_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04d')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f4_data))
      case (NF90_DOUBLE)
         allocate(f_md_obj%g_Att(i)%f8_data(f_md_obj%g_Att(i)%dimv), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #04e')
         call mNc_check(nf90_get_att(f_md_obj%ncid, NF90_GLOBAL,  &
              f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f8_data))
      end select

      f_md_obj%g_Att(i)%is_populated = .TRUE.
   end do
end if

!--- Obtain variable names and other information:

! Obtain some variable info; determine some maximum dimensions (per variable):

max_tmp1 = -999; max_tmp2 = -999
do i=1,f_md_obj%n_vars
   f_md_obj%varid(i) = i
   call mNc_check(nf90_inquire_variable(f_md_obj%ncid, f_md_obj%varid(i),  &
        f_md_obj%var_name(i), f_md_obj%var_type(i), f_md_obj%var_ndims(i),  &
        nAtts=f_md_obj%var_nAtts(i)))
   f_md_obj%var_name_len(i) = len_trim(f_md_obj%var_name(i))
   max_tmp1 = max(max_tmp1, f_md_obj%var_ndims(i))
   max_tmp2 = max(max_tmp2, f_md_obj%var_nAtts(i))
end do
f_md_obj%max_var_ndims = max_tmp1; f_md_obj%max_var_nAtts = max_tmp2

allocate(f_md_obj%var_dimids(f_md_obj%max_var_ndims, f_md_obj%n_vars), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'allocation #05')
allocate(f_md_obj%var_Att(f_md_obj%max_var_nAtts, f_md_obj%n_vars), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'allocation #06')

! Obtain dimension IDs and attribute information for each variable:

do i=1,f_md_obj%n_vars
   call mNc_check(nf90_inquire_variable(f_md_obj%ncid, f_md_obj%varid(i),  &
        dimids=f_md_obj%var_dimids(:,i)))
   do ia=1,f_md_obj%var_nAtts(i)
      f_md_obj%var_Att(ia,i)%id = ia
      call mNc_check(nf90_inq_attname(f_md_obj%ncid, f_md_obj%varid(i),  &
           f_md_obj%var_Att(ia,i)%id, f_md_obj%var_Att(ia,i)%name))
      call mNc_check(nf90_inquire_attribute(f_md_obj%ncid, f_md_obj%varid(i),  &
           f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%type,  &
           f_md_obj%var_Att(ia,i)%dimv))

      select case (f_md_obj%var_Att(ia,i)%type)
      case (NF90_BYTE)
         allocate(f_md_obj%var_Att(ia,i)%i1_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07a')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%i1_data))
      case (NF90_CHAR)
         allocate(f_md_obj%var_Att(ia,i)%c_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07aa')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, char_att_str_tmp))
         do ic=1,len_trim(char_att_str_tmp)
            f_md_obj%var_Att(ia,i)%c_data(ic) = char_att_str_tmp(ic:ic)
         end do
      case (NF90_SHORT)
         allocate(f_md_obj%var_Att(ia,i)%i2_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07b')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%i2_data))
      case (NF90_INT)
         allocate(f_md_obj%var_Att(ia,i)%i4_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07c')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%i4_data))
      case (NF90_FLOAT)
         allocate(f_md_obj%var_Att(ia,i)%f4_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07d')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%f4_data))
      case (NF90_DOUBLE)
         allocate(f_md_obj%var_Att(ia,i)%f8_data(f_md_obj%var_Att(ia,i)%dimv), &
              stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'allocation #07e')
         call mNc_check(nf90_get_att(f_md_obj%ncid, f_md_obj%varid(i),  &
              f_md_obj%var_Att(ia,i)%name, f_md_obj%var_Att(ia,i)%f8_data))
      end select

      f_md_obj%var_Att(ia,i)%is_populated = .TRUE.
   end do
end do

f_md_obj%is_populated = .TRUE.

end subroutine mNc_populate_f_md_obj_r


!```````````````````````````````````````````````````````````````````
subroutine mNc_populate_f_md_obj_w(nc_file_name, f_md_obj, n_dims, n_vars,  &
     n_g_Atts, dim_name, dimv, g_Att, var_name, var_type, var_ndims,  &
     var_nAtts, var_dimids, var_Att, open_existing_file)

!+ Populate the elements of an object of type 'mNc_file_metadata_obj' (from
!   arguments to this procedure), while also creating/defining a new NetCDF file
!
!  NOTE: An unlimited dimension is denoted by setting dimv(i) = -1
!
!  If only the first two arguments are supplied, creates a new file using the
!   metadata stored in 'f_md_obj'.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use netcdf, only: nf90_create, NF90_CLOBBER, NF90_MAX_NAME, nf90_def_dim,  &
     NF90_UNLIMITED, NF90_GLOBAL, NF90_BYTE, NF90_CHAR, NF90_SHORT, NF90_INT,  &
     NF90_FLOAT, NF90_DOUBLE, nf90_put_att, nf90_def_var, nf90_enddef,  &
     nf90_open, NF90_WRITE, nf90_inq_dimid

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: nc_file_name
integer, optional, intent(IN) :: n_dims, n_vars, n_g_Atts
character(len=*), optional, dimension(:), intent(IN) :: dim_name, var_name
integer, optional, dimension(:), intent(IN) :: dimv, var_type, var_ndims,  &
     var_nAtts
integer, optional, dimension(:,:), intent(IN) :: var_dimids
type(mNc_file_md_Att_obj), optional, dimension(:), intent(IN) :: g_Att
type(mNc_file_md_Att_obj), optional, dimension(:,:), intent(IN) :: var_Att
logical, optional, intent(IN) :: open_existing_file

type(mNc_file_metadata_obj), intent(INOUT) :: f_md_obj

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_populate_f_md_obj_w'

character(len=256) :: error_msg
integer :: ic, i, e, ia, max_tmp1, max_tmp2
logical :: md_was_provided, define_new_f

!=== Executable statements:

define_new_f = .TRUE.
if (present(open_existing_file)) define_new_f = .not.open_existing_file

md_was_provided = ( present(n_dims) .and. present(n_vars) .and.  &
     present(n_g_Atts) .and. present(dim_name) .and. present(dimv) .and.  &
     present(g_Att) .and. present(var_name) .and. present(var_type) .and.  &
     present(var_ndims) .and. present(var_nAtts) .and.  &
     present(var_dimids) .and. present(var_Att) )

if (md_was_provided .and. f_md_obj%is_populated) call exception(  &
     procedure_name, FATAL, 'This NetCDF file metadata object is already '//  &
     'populated; use `mNc_recycle_md_obj` first')

!--- Create a new NetCDF file or open an existing one:

ic = len_trim(nc_file_name)
f_md_obj%file_name = nc_file_name(1:ic)   ! Store filename string

if (ic > NF90_MAX_NAME) then   ! Check for NetCDF string length limit
   write (error_msg, '(a,i6,a)')  &
   'The given NetCDF file name is too long; must be <= ', NF90_MAX_NAME,  &
   ' characters in length'
   call exception(procedure_name, FATAL, error_msg)
end if

if (define_new_f) then
   call mNc_check(nf90_create(nc_file_name(1:ic), NF90_CLOBBER, f_md_obj%ncid))
else
   call mNc_check(nf90_open(nc_file_name(1:ic), NF90_WRITE, f_md_obj%ncid))
end if

!--- Define/set important file parameters; allocate relevant object elements:

if (md_was_provided) then

   if (n_dims < 1 .or. n_vars < 1) call exception(procedure_name,  &
        FATAL, 'Arguments n_dims and n_vars must both be greater than zero.')

   f_md_obj%n_dims = n_dims; f_md_obj%n_vars = n_vars
   f_md_obj%n_g_Atts = n_g_Atts

   allocate(f_md_obj%dimid(f_md_obj%n_dims),  &
        f_md_obj%dim_name(f_md_obj%n_dims), f_md_obj%dimv(f_md_obj%n_dims),  &
        stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01')
   allocate(f_md_obj%varid(f_md_obj%n_vars),  &
        f_md_obj%var_name(f_md_obj%n_vars),  &
        f_md_obj%var_type(f_md_obj%n_vars),  &
        f_md_obj%var_ndims(f_md_obj%n_vars),  &
        f_md_obj%var_nAtts(f_md_obj%n_vars),  &
        f_md_obj%var_name_len(f_md_obj%n_vars), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #02')
   if (f_md_obj%n_g_Atts > 0) then
      allocate(f_md_obj%g_Att(f_md_obj%n_g_Atts), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'allocation #03')
   end if

   !--- Define/set dimension sizes and names:

   i = min(size(dim_name, 1), size(dimv, 1))
   if (i < f_md_obj%n_dims) call exception(procedure_name, FATAL,  &
        'Dimensions of arguments dim_name and dimv must both be >= n_dims.')

   f_md_obj%unlimited_dim_exists = .FALSE.
   do i=1,f_md_obj%n_dims
      ic = min(NF90_MAX_NAME, len_trim(dim_name(i)))
      f_md_obj%dim_name(i) = dim_name(i)(1:ic)

      if (dimv(i) /= -1) then
         if (dimv(i) < 1) then
            write (error_msg, '(a,i3,a)')  &
                 'An improper value was given for for dimension #', i,  &
                 '; change it and retry.'
            call exception(procedure_name, FATAL, error_msg)
         end if
         f_md_obj%dimv(i) = dimv(i)
         if (define_new_f) then
            call mNc_check(nf90_def_dim(f_md_obj%ncid, f_md_obj%dim_name(i),  &
                 f_md_obj%dimv(i), f_md_obj%dimid(i)))
         else
            call mNc_check(nf90_inq_dimid(f_md_obj%ncid,  &
                 f_md_obj%dim_name(i), f_md_obj%dimid(i)))
         end if
      else    ! The unlimited dimension
         if (f_md_obj%unlimited_dim_exists) then
            write (error_msg, '(a,i3,a)')  &
                 'Dimension #', i, ' is indicated as a second unlimited '// &
                 'dimension; only one is permitted.'
            call exception(procedure_name, FATAL, error_msg)
         end if
         f_md_obj%dimv(i) = dimv(i)
         if (define_new_f) then
            call mNc_check(nf90_def_dim(f_md_obj%ncid, f_md_obj%dim_name(i),  &
                 NF90_UNLIMITED, f_md_obj%dimid(i)))
         else
            call mNc_check(nf90_inq_dimid(f_md_obj%ncid,  &
                 f_md_obj%dim_name(i), f_md_obj%dimid(i)))
         end if
         f_md_obj%unlimited_dimid = f_md_obj%dimid(i)
         f_md_obj%unlimited_dim_exists = .TRUE.
      end if
   end do

else if (define_new_f) then  !*** Fill from already-populated file metadata obj

   !--- Define/set dimension sizes and names:

   do i=1,f_md_obj%n_dims
      if (f_md_obj%dimv(i) /= -1) then
         call mNc_check(nf90_def_dim(f_md_obj%ncid, f_md_obj%dim_name(i),  &
              f_md_obj%dimv(i), f_md_obj%dimid(i)))
      else    ! The unlimited dimension
         call mNc_check(nf90_def_dim(f_md_obj%ncid, f_md_obj%dim_name(i),  &
              NF90_UNLIMITED, f_md_obj%dimid(i)))
      end if
   end do

end if

if (md_was_provided) then

   !--- Define/set global attribute information (if any):

   if (f_md_obj%n_g_Atts > 0) then
      if (size(g_Att, 1) < f_md_obj%n_g_Atts) call exception(procedure_name,  &
           FATAL, 'Dimension of argument g_Att must be >= n_g_Atts.')

      do i=1,f_md_obj%n_g_Atts
         call mNc_copy_f_md_Att_obj(g_Att(i), f_md_obj%g_Att(i))

         if (define_new_f) then
            select case (f_md_obj%g_Att(i)%type)
            case (NF90_BYTE)
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i1_data))
            case (NF90_CHAR)
               char_att_str_tmp = ';'
               do ic=1,min(f_md_obj%g_Att(i)%dimv, mNc_MAX_CHAR_ATT_LEN)
                  char_att_str_tmp(ic:ic) = f_md_obj%g_Att(i)%c_data(ic)
               end do
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, char_att_str_tmp))
            case (NF90_SHORT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i2_data))
            case (NF90_INT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i4_data))
            case (NF90_FLOAT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f4_data))
            case (NF90_DOUBLE)
               call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                    f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f8_data))
            end select
         end if
      end do
   end if

else if (define_new_f) then !*** Fill from already-populated file metadata obj

   !--- Define/set global attribute information (if any):

   if (f_md_obj%n_g_Atts > 0) then
      do i=1,f_md_obj%n_g_Atts
         select case (f_md_obj%g_Att(i)%type)
         case (NF90_BYTE)
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i1_data))
         case (NF90_CHAR)
            char_att_str_tmp = ';'
            do ic=1,min(f_md_obj%g_Att(i)%dimv, mNc_MAX_CHAR_ATT_LEN)
               char_att_str_tmp(ic:ic) = f_md_obj%g_Att(i)%c_data(ic)
            end do
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, char_att_str_tmp))
         case (NF90_SHORT)
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i2_data))
         case (NF90_INT)
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%i4_data))
         case (NF90_FLOAT)
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f4_data))
         case (NF90_DOUBLE)
            call mNc_check(nf90_put_att(f_md_obj%ncid, NF90_GLOBAL,  &
                 f_md_obj%g_Att(i)%name, f_md_obj%g_Att(i)%f8_data))
         end select
      end do
   end if

end if

if (md_was_provided) then

   !--- Define/set variable names and other information:

   i = min(size(var_name, 1), size(var_type, 1), size(var_ndims, 1),  &
        size(var_nAtts, 1))
   if (i < f_md_obj%n_vars) call exception(procedure_name, FATAL,  &
        'First dimensions of arguments var_* (except var_Att, var_dimids) '//  &
        'must all be >= n_vars.')

   if (size(var_dimids, 2) < f_md_obj%n_vars) call exception(procedure_name,  &
        FATAL, 'Second dimension of argument var_dimids must be >= n_vars.')

   if (minval(var_nAtts) > 0) then
      if (size(var_Att, 2) < f_md_obj%n_vars) call exception(procedure_name,  &
           FATAL, 'Second dimension of argument var_Att must be >= n_vars.')
   end if

   max_tmp1 = -999; max_tmp2 = -999
   do i=1,f_md_obj%n_vars
      f_md_obj%varid(i) = i
      
      ic = min(NF90_MAX_NAME, len_trim(var_name(i)))
      f_md_obj%var_name(i) = var_name(i)(1:ic)
      f_md_obj%var_name_len(i) = ic

      f_md_obj%var_type(i) = var_type(i)
      f_md_obj%var_ndims(i) = var_ndims(i)
      f_md_obj%var_nAtts(i) = var_nAtts(i)
   
      max_tmp1 = max(max_tmp1, f_md_obj%var_ndims(i))
      max_tmp2 = max(max_tmp2, f_md_obj%var_nAtts(i))
   end do
   f_md_obj%max_var_ndims = max_tmp1; f_md_obj%max_var_nAtts = max_tmp2

   allocate(f_md_obj%var_dimids(f_md_obj%max_var_ndims, f_md_obj%n_vars),  &
        stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #04')
   allocate(f_md_obj%var_Att(f_md_obj%max_var_nAtts, f_md_obj%n_vars), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #05')

   do i=1,f_md_obj%n_vars
      ! Process dimension IDs for this variable:

      if (size(var_dimids, 1) < f_md_obj%var_ndims(i)) call exception(  &
           procedure_name, FATAL,  &
           'First dimension of argument var_dimids must be >= var_ndims(i).')

      ia = f_md_obj%var_ndims(i)
      f_md_obj%var_dimids(1:ia,i) = var_dimids(1:ia,i)

      if (define_new_f) call mNc_check(nf90_def_var(f_md_obj%ncid,  &
           f_md_obj%var_name(i), f_md_obj%var_type(i),  &
           pack(var_dimids(1:ia,i), .TRUE.), f_md_obj%varid(i)))

      ! Process any attributes for this variable
   
      if (f_md_obj%var_nAtts(i) > 0) then
         if (size(var_Att, 1) < f_md_obj%var_nAtts(i)) call exception(  &
              procedure_name, FATAL,  &
              'First dimension of argument var_Att must be >= var_nAtts(i).')

         do ia=1,f_md_obj%var_nAtts(i)
            call mNc_copy_f_md_Att_obj(var_Att(ia,i), f_md_obj%var_Att(ia,i))

            if (define_new_f) then
               select case (f_md_obj%var_Att(ia,i)%type)
               case (NF90_BYTE)
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       f_md_obj%var_Att(ia,i)%i1_data))
               case (NF90_CHAR)
                  char_att_str_tmp = ';'
                  do ic=1,min(f_md_obj%var_Att(ia,i)%dimv, mNc_MAX_CHAR_ATT_LEN)
                     char_att_str_tmp(ic:ic) = f_md_obj%var_Att(ia,i)%c_data(ic)
                  end do
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       char_att_str_tmp))
               case (NF90_SHORT)
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       f_md_obj%var_Att(ia,i)%i2_data))
               case (NF90_INT)
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       f_md_obj%var_Att(ia,i)%i4_data))
               case (NF90_FLOAT)
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       f_md_obj%var_Att(ia,i)%f4_data))
               case (NF90_DOUBLE)
                  call mNc_check(nf90_put_att(f_md_obj%ncid,  &
                       f_md_obj%varid(i), f_md_obj%var_Att(ia,i)%name,  &
                       f_md_obj%var_Att(ia,i)%f8_data))
               end select
            end if
         end do
      end if
   end do

else if (define_new_f) then !*** Fill from already-populated file metadata obj

   do i=1,f_md_obj%n_vars
      ! Process dimension IDs for this variable:

      ia = f_md_obj%var_ndims(i)

      call mNc_check(nf90_def_var(f_md_obj%ncid, f_md_obj%var_name(i),  &
           f_md_obj%var_type(i), pack(f_md_obj%var_dimids(1:ia,i), .TRUE.),  &
           f_md_obj%varid(i)))

      ! Process any attributes for this variable
   
      if (f_md_obj%var_nAtts(i) > 0) then

         do ia=1,f_md_obj%var_nAtts(i)
            select case (f_md_obj%var_Att(ia,i)%type)
            case (NF90_BYTE)
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name,  &
                    f_md_obj%var_Att(ia,i)%i1_data))
            case (NF90_CHAR)
               char_att_str_tmp = ';'
               do ic=1,min(f_md_obj%var_Att(ia,i)%dimv, mNc_MAX_CHAR_ATT_LEN)
                  char_att_str_tmp(ic:ic) = f_md_obj%var_Att(ia,i)%c_data(ic)
               end do
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name, char_att_str_tmp))
            case (NF90_SHORT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name,  &
                    f_md_obj%var_Att(ia,i)%i2_data))
            case (NF90_INT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name,  &
                    f_md_obj%var_Att(ia,i)%i4_data))
            case (NF90_FLOAT)
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name,  &
                    f_md_obj%var_Att(ia,i)%f4_data))
            case (NF90_DOUBLE)
               call mNc_check(nf90_put_att(f_md_obj%ncid, f_md_obj%varid(i),  &
                    f_md_obj%var_Att(ia,i)%name,  &
                    f_md_obj%var_Att(ia,i)%f8_data))
            end select
         end do
      end if
   end do

end if

! Leave DEFINE mode, open new file, write metadata, enter DATA mode:
if (define_new_f) call mNc_check(nf90_enddef(f_md_obj%ncid))

f_md_obj%is_populated = .TRUE.

end subroutine mNc_populate_f_md_obj_w


!```````````````````````````````````````````````````````````````````
subroutine mNc_recycle_f_md_Att_obj(f_md_Att_obj)

!+ "Recycle" the elements of an object of type 'mNc_file_md_Att_obj' (i.e.,
!   deallocate dynamic memory, nullify pointers)

use univ_exception_mod, only: exception, WARN
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_deallocation

implicit none

!=== Argument declarations:

type(mNc_file_md_Att_obj), intent(INOUT) :: f_md_Att_obj

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_recycle_f_md_Att_obj'

integer :: e

!=== Executable statements:

if (.NOT.f_md_Att_obj%is_populated) then
   call exception(procedure_name, WARN,  &
        'This NetCDF attribute object is not populated; nothing to do')
   return
end if

! Deallocate dynamic memory:

e = 0
if (associated(f_md_Att_obj%i1_data)) then
   deallocate(f_md_Att_obj%i1_data, stat=e)
else if (associated(f_md_Att_obj%i2_data)) then
   deallocate(f_md_Att_obj%i2_data, stat=e)
else if (associated(f_md_Att_obj%i4_data)) then
   deallocate(f_md_Att_obj%i4_data, stat=e)
else if (associated(f_md_Att_obj%f4_data)) then
   deallocate(f_md_Att_obj%f4_data, stat=e)
else if (associated(f_md_Att_obj%f8_data)) then
   deallocate(f_md_Att_obj%f8_data, stat=e)
end if
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'deallocation #01')

nullify(f_md_Att_obj%i1_data); nullify(f_md_Att_obj%c_data)
nullify(f_md_Att_obj%i2_data); nullify(f_md_Att_obj%i4_data)
nullify(f_md_Att_obj%f4_data); nullify(f_md_Att_obj%f8_data)

f_md_Att_obj%name = ';'

f_md_Att_obj%is_populated = .FALSE.

end subroutine mNc_recycle_f_md_Att_obj


!```````````````````````````````````````````````````````````````````
subroutine mNc_recycle_f_md_obj(f_md_obj)

!+ "Recycle" the elements of an object of type 'mNc_file_metadata_obj' (i.e.,
!   deallocate dynamic memory, nullify pointers, and close any associated
!   NetCDF file)

use univ_exception_mod, only: exception, WARN
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_deallocation
use netcdf, only: nf90_close

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(INOUT) :: f_md_obj

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_recycle_f_md_obj'

integer :: e, i, j

!=== Executable statements:

if (.NOT.f_md_obj%is_populated) then
   call exception(procedure_name, WARN,  &
        'This NetCDF file metadata object is not populated; nothing to do')
   return
end if

! Deallocate dynamic memory:

if (associated(f_md_obj%var_Att)) then
   do j=1,size(f_md_obj%var_Att,2)
      do i=1,size(f_md_obj%var_Att,1)
         if (f_md_obj%var_Att(i,j)%is_populated)  &
              call mNc_recycle_f_md_Att_obj(f_md_obj%var_Att(i,j))
      end do
   end do
   deallocate(f_md_obj%var_Att, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'deallocation #01')
   nullify(f_md_obj%var_Att)
end if

if (associated(f_md_obj%var_dimids)) then
   deallocate(f_md_obj%var_dimids, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'deallocation #02')
   nullify(f_md_obj%var_dimids)
end if

if (associated(f_md_obj%g_Att)) then
   do i=1,size(f_md_obj%g_Att,1)
      if (f_md_obj%g_Att(i)%is_populated)  &
           call mNc_recycle_f_md_Att_obj(f_md_obj%g_Att(i))
   end do
   deallocate(f_md_obj%g_Att, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'deallocation #03')
end if

if (associated(f_md_obj%varid)) then
   deallocate(f_md_obj%varid, f_md_obj%var_name, f_md_obj%var_type,  &
        f_md_obj%var_ndims, f_md_obj%var_nAtts, f_md_obj%var_name_len, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'deallocation #04')
   nullify(f_md_obj%varid, f_md_obj%var_name, f_md_obj%var_type,  &
        f_md_obj%var_ndims, f_md_obj%var_nAtts, f_md_obj%var_name_len)
end if

if (associated(f_md_obj%dimid)) then
   deallocate(f_md_obj%dimid, f_md_obj%dim_name, f_md_obj%dimv, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'deallocation #05')
   nullify(f_md_obj%dimid, f_md_obj%dim_name, f_md_obj%dimv)
end if

!--- Close the NetCDF file:
call mNc_close(f_md_obj)

f_md_obj%is_populated = .FALSE.

end subroutine mNc_recycle_f_md_obj


!```````````````````````````````````````````````````````````````````
subroutine mNc_copy_f_md_Att_obj(f_md_Att_obj, f_md_Att_obj_copy)

!+ Populate the elements of an object of type 'mNc_file_md_Att_obj', using
!   data from an existing 'mNc_file_md_Att_obj' object.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use netcdf, only: NF90_BYTE, NF90_CHAR, NF90_SHORT, NF90_INT, NF90_FLOAT,  &
     NF90_DOUBLE

implicit none

!=== Argument declarations:

type(mNc_file_md_Att_obj), intent(IN) :: f_md_Att_obj

type(mNc_file_md_Att_obj), intent(INOUT) :: f_md_Att_obj_copy

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_copy_f_md_Att_obj'

integer :: e

!=== Executable statements:

if (f_md_Att_obj_copy%is_populated) call exception(procedure_name, FATAL,  &
     'The attribute object (will be the copy) is already populated; use '//  &
     '`mNc_recycle_md_Att_obj` first')

if (.NOT.f_md_Att_obj%is_populated) call exception(procedure_name,  &
     FATAL, 'The attribute object (to be copied) is not populated')

f_md_Att_obj_copy%name = f_md_Att_obj%name
f_md_Att_obj_copy%id = f_md_Att_obj%id
f_md_Att_obj_copy%type = f_md_Att_obj%type
f_md_Att_obj_copy%dimv = f_md_Att_obj%dimv

select case (f_md_Att_obj%type)
case (NF90_BYTE)
   allocate(f_md_Att_obj_copy%i1_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01a')
   f_md_Att_obj_copy%i1_data = f_md_Att_obj%i1_data
case (NF90_CHAR)
   allocate(f_md_Att_obj_copy%c_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01aa')
   f_md_Att_obj_copy%c_data = f_md_Att_obj%c_data
case (NF90_SHORT)
   allocate(f_md_Att_obj_copy%i2_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01b')
   f_md_Att_obj_copy%i2_data = f_md_Att_obj%i2_data
case (NF90_INT)
   allocate(f_md_Att_obj_copy%i4_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01c')
   f_md_Att_obj_copy%i4_data = f_md_Att_obj%i4_data
case (NF90_FLOAT)
   allocate(f_md_Att_obj_copy%f4_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01d')
   f_md_Att_obj_copy%f4_data = f_md_Att_obj%f4_data
case (NF90_DOUBLE)
   allocate(f_md_Att_obj_copy%f8_data(f_md_Att_obj_copy%dimv), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'allocation #01e')
   f_md_Att_obj_copy%f8_data = f_md_Att_obj%f8_data
end select

f_md_Att_obj_copy%is_populated = .TRUE.

end subroutine mNc_copy_f_md_Att_obj


!```````````````````````````````````````````````````````````````````
function mNc_var_ndims_n(f_md_obj, var_name, query_nc_f)  result(var_ndims)

!+ Return the number of dimensions that the specified variable has (supply the
!   variable name).
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: nf90_inquire_variable

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: var_name
logical, optional, intent(IN) :: query_nc_f

integer :: var_ndims      ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_ndims_n'

integer :: var_idx
logical :: query_f

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

query_f = .FALSE.
if (present(query_nc_f)) query_f = query_nc_f

var_idx = mNc_var_idx(f_md_obj, var_name, query_nc_f=query_f)

if (query_f) then
   call mNc_check(nf90_inquire_variable(f_md_obj%ncid, var_idx,  &
        ndims=var_ndims))
else
   var_ndims = f_md_obj%var_ndims(var_idx)
end if

end function mNc_var_ndims_n


!```````````````````````````````````````````````````````````````````
function mNc_var_ndims_i(f_md_obj, var_idx, query_nc_f)  result(var_ndims)

!+ Return the number of dimensions that the specified variable has (supply the
!   variable index).
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: nf90_inquire_variable

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: var_idx
logical, optional, intent(IN) :: query_nc_f

integer :: var_ndims      ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_ndims_i'

logical :: query_f

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

query_f = .FALSE.
if (present(query_nc_f)) query_f = query_nc_f

if (query_f) then
   call mNc_check(nf90_inquire_variable(f_md_obj%ncid, var_idx,  &
        ndims=var_ndims))
else
   var_ndims = f_md_obj%var_ndims(var_idx)
end if

end function mNc_var_ndims_i


!```````````````````````````````````````````````````````````````````
function mNc_g_Att_exists(f_md_obj, Att_name, query_nc_f)  result(exists)

!+ Does the queried global attribute within a NetCDF file metadata object exist?
!
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: NF90_MAX_NAME, nf90_inquire_attribute, NF90_GLOBAL, NF90_NOERR

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: Att_name
logical, optional, intent(IN) :: query_nc_f

logical :: exists     ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_g_Att_exists'

integer :: i, ic, idx
logical :: query_f_md_obj

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

query_f_md_obj = .TRUE.
if (present(query_nc_f)) query_f_md_obj = .not.query_nc_f

ic = min(len_trim(Att_name), NF90_MAX_NAME)   ! Truncate name to maximum length
if (query_f_md_obj) then
   idx = -1
   do i=1,f_md_obj%n_g_Atts
      if (trim(f_md_obj%g_Att(i)%name) == Att_name(1:ic)) then
         idx = i; exit
      end if
   end do
   exists = ( idx /= -1 )
else
   i = nf90_inquire_attribute(f_md_obj%ncid, NF90_GLOBAL, Att_name(1:ic))
   exists = ( i == NF90_NOERR )
end if

end function mNc_g_Att_exists


!```````````````````````````````````````````````````````````````````
function mNc_var_Att_exists(f_md_obj, var_name, Att_name,  &
     query_nc_f)  result(exists)

!+ Does the queried variable attribute within a NetCDF file metadata object
!   exist?
!
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: NF90_MAX_NAME, nf90_inquire_attribute, NF90_NOERR

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: var_name, Att_name
logical, optional, intent(IN) :: query_nc_f

logical :: exists     ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_Att_exists'

integer :: i, ic, idx, var_idx
logical :: query_f

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

query_f = .FALSE.
if (present(query_nc_f)) query_f = query_nc_f

var_idx = mNc_var_idx(f_md_obj, var_name, neg1_when_not_found=.TRUE.,  &
     query_nc_f=query_f)
exists = ( var_idx > 0 )

if (exists) then
   ic = min(len_trim(Att_name), NF90_MAX_NAME) ! Truncate name to maximum length
   if (query_f) then
      i = nf90_inquire_attribute(f_md_obj%ncid, var_idx, Att_name(1:ic))
      exists = ( i == NF90_NOERR )
   else
      idx = -1
      do i=1,f_md_obj%var_nAtts(var_idx)
         if (trim(f_md_obj%var_Att(i,var_idx)%name) == Att_name(1:ic)) then
            idx = i; exit
         end if
      end do
      exists = ( idx /= -1 )
   end if
end if

end function mNc_var_Att_exists


!```````````````````````````````````````````````````````````````````
function mNc_var_idx(f_md_obj, var_name, neg1_when_not_found,  &
     query_nc_f)  result(var_idx)

!+ Return the index of a variable (corresponding to the given variable name)
!   within a NetCDF file metadata object.
!
!  If 'neg1_when_not_found' is present and .TRUE., returns -1 if the queried
!   variable was not found.
!
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: NF90_MAX_NAME, NF90_NOERR, nf90_inq_varid

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: var_name
logical, optional, intent(IN) :: neg1_when_not_found, query_nc_f

integer :: var_idx     ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_idx'

integer :: i, ic
logical :: query_f_md_obj, fatal_when_not_found

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

fatal_when_not_found = .TRUE.
if (present(neg1_when_not_found))  &
     fatal_when_not_found = .not.neg1_when_not_found

query_f_md_obj = .TRUE.
if (present(query_nc_f)) query_f_md_obj = .not.query_nc_f

ic = min(len_trim(var_name), NF90_MAX_NAME)   ! Truncate name to maximum length
if (query_f_md_obj) then
   var_idx = -1
   do i=1,f_md_obj%n_vars
      if (f_md_obj%var_name_len(i) == ic) then
         if (f_md_obj%var_name(i)(1:f_md_obj%var_name_len(i)) ==  &
              var_name(1:ic)) then
            var_idx = i; exit
         end if
      end if
   end do
   if (var_idx == -1 .and. fatal_when_not_found) call exception(  &
        procedure_name, FATAL, 'Queried variable '//var_name(1:ic)//  &
        ' not found.')
else
   if (fatal_when_not_found) then
      call mNc_check(nf90_inq_varid(f_md_obj%ncid, var_name(1:ic), var_idx))
   else
      i = nf90_inq_varid(f_md_obj%ncid, var_name(1:ic), var_idx)
      if (i /= NF90_NOERR) var_idx = -1
   end if
end if

end function mNc_var_idx


!```````````````````````````````````````````````````````````````````
function mNc_dim_idx(f_md_obj, dim_name, neg1_when_not_found,  &
     query_nc_f)  result(dim_idx)

!+ Return the index of a file dimension (corresponding to the given dimension
!   name) within a NetCDF file metadata object.
!
!  If 'neg1_when_not_found' is present and .TRUE., returns -1 if the queried
!   dimension was not found.
!
!  If 'query_nc_f' is present and .TRUE., use NetCDF API routines whenever
!   possible to query the associated NetCDF file directly (instead of the file
!   metadata object) for the desired information.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use netcdf, only: NF90_MAX_NAME, NF90_NOERR, nf90_inq_dimid

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: dim_name
logical, optional, intent(IN) :: neg1_when_not_found, query_nc_f

integer :: dim_idx     ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_dim_idx'

integer :: i, ic
logical :: query_f_md_obj, fatal_when_not_found

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

fatal_when_not_found = .TRUE.
if (present(neg1_when_not_found))  &
     fatal_when_not_found = .not.neg1_when_not_found

query_f_md_obj = .TRUE.
if (present(query_nc_f)) query_f_md_obj = .not.query_nc_f

ic = min(len_trim(dim_name), NF90_MAX_NAME)   ! Truncate name to maximum length
if (query_f_md_obj) then
   dim_idx = -1
   do i=1,f_md_obj%n_dims
      if (trim(f_md_obj%dim_name(i)) == dim_name(1:ic)) then
         dim_idx = i; exit
      end if
   end do
   if (dim_idx == -1 .and. fatal_when_not_found) call exception(  &
        procedure_name, FATAL, 'Queried dimension '//dim_name(1:ic)//  &
        ' not found.')
else
   if (fatal_when_not_found) then
      call mNc_check(nf90_inq_dimid(f_md_obj%ncid, dim_name(1:ic), dim_idx))
   else
      i = nf90_inq_dimid(f_md_obj%ncid, dim_name(1:ic), dim_idx)
      if (i /= NF90_NOERR) dim_idx = -1
   end if
end if

end function mNc_dim_idx


!```````````````````````````````````````````````````````````````````
function mNc_var_dims_i(f_md_obj, var_idx, n_vd)  result(dim_values)

!+ Returns the corresponding dimension values (lengths) for a specified
!   variable, given a NetCDF file metadata object, the index of a variable, and
!   the number of dimensions that variable has (needed for automatic allocation
!   of the function result).

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: var_idx, n_vd

integer, dimension(n_vd) :: dim_values  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_dims_i'

integer :: iq, i

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Check internal and given variable dimensions for consistency:
if (f_md_obj%var_ndims(var_idx) /= n_vd) call exception(procedure_name,  &
     FATAL, 'Internal and given number of dimensions for variable ('//  &
     f_md_obj%var_name(var_idx)(1:f_md_obj%var_name_len(var_idx))//  &
     ') do not match.')

! Access dimension values:
do i=1,n_vd
   iq = f_md_obj%var_dimids(i,var_idx)
   dim_values(i) = f_md_obj%dimv(iq)
end do

end function mNc_var_dims_i


!```````````````````````````````````````````````````````````````````
function mNc_var_dims_n(f_md_obj, var_name, n_vd)  result(dim_values)

!+ Returns the corresponding dimension values (lengths) for a specified
!   variable, given a NetCDF file metadata object, the name of a variable, and
!   the number of dimensions that variable has (needed for automatic allocation
!   of the function result).

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: var_name
integer, intent(IN) :: n_vd

integer, dimension(n_vd) :: dim_values  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_dims_n'

integer :: iq, i, var_idx

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Determine the variable's index:
var_idx = mNc_var_idx(f_md_obj, var_name)

! Check internal and given variable dimensions for consistency:
if (f_md_obj%var_ndims(var_idx) /= n_vd) call exception(procedure_name,  &
     FATAL, 'Internal and given number of dimensions for variable ('//  &
     f_md_obj%var_name(var_idx)(1:f_md_obj%var_name_len(var_idx))//  &
     ') do not match.')

! Access dimension values:
do i=1,n_vd
   iq = f_md_obj%var_dimids(i,var_idx)
   dim_values(i) = f_md_obj%dimv(iq)
end do

end function mNc_var_dims_n


!```````````````````````````````````````````````````````````````````
function mNc_var_dimval_i(f_md_obj, var_idx, dim_idx)  result(dim_value)

!+ Returns the desired dimension value (length) for a specified variable, given
!   a NetCDF file metadata object, the index of a variable, and the dimension
!   index.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: var_idx, dim_idx

integer :: dim_value  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_dimval_i'

integer :: i

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Access dimension value:
i = f_md_obj%var_dimids(dim_idx,var_idx)
dim_value = f_md_obj%dimv(i)

end function mNc_var_dimval_i


!```````````````````````````````````````````````````````````````````
function mNc_var_dimval_n(f_md_obj, var_name, dim_idx)  result(dim_value)

!+ Returns the desired dimension value (length) for a specified variable, given
!   a NetCDF file metadata object, the name of a variable, and the dimension
!   index.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: var_name
integer, intent(IN) :: dim_idx

integer :: dim_value  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_var_dimval_n'

integer :: i, var_idx

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Determine the variable's index:
var_idx = mNc_var_idx(f_md_obj, var_name)

! Access dimension values:
i = f_md_obj%var_dimids(dim_idx,var_idx)
dim_value = f_md_obj%dimv(i)

end function mNc_var_dimval_n


!```````````````````````````````````````````````````````````````````
function mNc_dim_val_i(f_md_obj, dim_idx)  result(dim_value)

!+ Returns the corresponding dimension value (length) for a specified
!   dimension, given a NetCDF file metadata object and the dimension's index.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: dim_idx

integer :: dim_value  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_dim_val_i'

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Access dimension value:
dim_value = f_md_obj%dimv(dim_idx)

end function mNc_dim_val_i


!```````````````````````````````````````````````````````````````````
function mNc_dim_val_n(f_md_obj, dim_name)  result(dim_value)

!+ Returns the corresponding dimension value (length) for a specified
!   dimension, given a NetCDF file metadata object and the dimension's name.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
character(len=*), intent(IN) :: dim_name

integer :: dim_value  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_dim_val_n'

integer :: dim_idx

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

! Access dimension value:
dim_idx = mNc_dim_idx(f_md_obj, dim_name)
dim_value = f_md_obj%dimv(dim_idx)

end function mNc_dim_val_n


!```````````````````````````````````````````````````````````````````
function mNc_f_md_populated(f_md_obj)  result(populated)

!+ Returns the 'is_populated' component of a given NetCDF file metadata object.

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj

logical :: populated  ! Function result dec

!== Local declarations:

! NONE

!=== Executable statements:

populated = f_md_obj%is_populated

end function mNc_f_md_populated


!```````````````````````````````````````````````````````````````````
function mNc_f_md_Att_populated(f_md_Att_obj)  result(populated)

!+ Returns the 'is_populated' component of a given NetCDF file attribute
!   metadata object.

implicit none

!=== Argument declarations:

type(mNc_file_md_Att_obj), intent(IN) :: f_md_Att_obj

logical :: populated  ! Function result dec

!== Local declarations:

! NONE

!=== Executable statements:

populated = f_md_Att_obj%is_populated

end function mNc_f_md_Att_populated


!```````````````````````````````````````````````````````````````````
function mNc_f_md_var_type(f_md_obj, var_idx)  result(v_type)

!+ Return the 'var_type' component for a given variable index within a given
!   NetCDF file metadata object.  Returns -1 if the var_idx is invalid.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: var_idx

integer :: v_type     ! Function result declaration

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_f_md_var_type'

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

if (var_idx >= 1 .and. var_idx <= f_md_obj%n_vars) then
   v_type = f_md_obj%var_type(var_idx)
else
   v_type = -1
end if

end function mNc_f_md_var_type


!```````````````````````````````````````````````````````````````````
subroutine mNc_sync_with_file(f_md_obj)

!+ Syncs internal read/write buffers with an open NetCDF file

use netcdf, only: nf90_sync

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj

!== Local declarations:

! NONE

!=== Executable statements:

call mNc_check(nf90_sync(f_md_obj%ncid))

end subroutine mNc_sync_with_file


!```````````````````````````````````````````````````````````````````
subroutine mNc_close(f_md_obj)

!+ Given a NetCDF file metadata object, closes its associated NetCDF file.

use netcdf, only: nf90_close

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj

!== Local declarations:

! NONE

!=== Executable statements:

call mNc_check(nf90_close(f_md_obj%ncid))

end subroutine mNc_close


!```````````````````````````````````````````````````````````````````
subroutine mNc_check(status, caller_ID_str)

!+ Checks NetCDF procedure status

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use netcdf, only: nf90_noerr, nf90_strerror

implicit none

!=== Argument declarations:

integer, intent(IN) :: status
character(len=*), optional, intent(IN) :: caller_ID_str

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_check'

character(len=MAX_FILE_PATH_LEN) :: str_tmp

!=== Executable statements:

if (present(caller_ID_str)) then
   str_tmp = trim(procedure_name)//trim(caller_ID_str)
else
   str_tmp = trim(procedure_name)
end if

if (status /= nf90_noerr) call exception(str_tmp, FATAL,  &
     trim(nf90_strerror(status)))

end subroutine mNc_check


!```````````````````````````````````````````````````````````````````
function mNc_file_ID(f_md_obj)  result(ID)

!+ Returns the 'ncid' component of a given NetCDF file metadata object.

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj

integer :: ID  ! Function result dec

!== Local declarations:

! NONE

!=== Executable statements:

ID = f_md_obj%ncid

end function mNc_file_ID


!```````````````````````````````````````````````````````````````````
function mNc_is_the_unlimited_dim(f_md_obj, var_idx, dim_idx)  result(answer)

!+ Returns whether the specified dimension of a variable is the special
!   "unlimited" dimension, given a NetCDF file metadata object, the index of a
!   variable, and the variable's dimension index to be queried.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

type(mNc_file_metadata_obj), intent(IN) :: f_md_obj
integer, intent(IN) :: var_idx, dim_idx

logical :: answer  ! Function result dec

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mNc_is_the_unlimited_dim'

!=== Executable statements:

! Abort if file metadata object is empty:
if (.NOT.f_md_obj%is_populated) call exception(procedure_name, FATAL,  &
     'The given NetCDF file metadata object is not populated.')

answer = ( f_md_obj%unlimited_dim_exists .and.  &
           f_md_obj%var_dimids(dim_idx,var_idx) == f_md_obj%unlimited_dimid )

end function mNc_is_the_unlimited_dim


end module manage_NetCDF_classic_mod
