"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict( \
                   use_kd_mod = 'use univ_kind_defs_mod, only:' \
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict( \
        kind_idx_n = ['i1', 'c', 'i2', 'i4', 'f4', 'f8'], \
        kind_idx_use = ['i1', ' ', 'i2', 'i4', 'f4', 'f8'], \
        kind_idx_dec = ['i1', 'len=*', 'i2', 'i4', 'f4', 'f8'], \
        dec_dim = [' dimension(:),', '', ' dimension(:),', \
                   ' dimension(:),', ' dimension(:),', ' dimension(:),'], \
        kind_desc = ['1-byte integer', 'character', '2-byte integer', \
                     '4-byte integer', '4-byte floating point', \
                     '8-byte floating point'], \
        kind_mod = ['${use_kd_mod}', ' ', '${use_kd_mod}', '${use_kd_mod}', \
                    '${use_kd_mod}', '${use_kd_mod}'], \
        nc_kind = ['NF90_BYTE', 'NF90_CHAR', 'NF90_SHORT', 'NF90_INT', \
                   'NF90_FLOAT', 'NF90_DOUBLE'], \
        kind_type = ['integer', 'character', 'integer', 'integer', 'real', \
                     'real'], \
        c_len_loop_beg = [' ', 'do ic=1,min(Att_dimv, mNc_MAX_CHAR_ATT_LEN)', \
                          ' ', ' ', ' ', ' '], \
        c_len_loop_end = [' ', 'end do', ' ', ' ', ' ', ' '], \
        c_len_chk1 = [' ', \
            'if (Att_dimv /= len_trim(Att_data)) call exception(procedure_name, FATAL,  &', \
                     ' ', ' ', ' ', ' '], \
        c_len_chk2 = [' ', \
            '     \"Given character attribute dim /= length of input string\")', \
                     ' ', ' ', ' ', ' '], \
        a_size_empty = ['size(Att_data)', 'len(Att_data)', 'size(Att_data)', \
                  'size(Att_data)', 'size(Att_data)', 'size(Att_data)'], \
        a_rem = ['(:)', '(ic)', '(:)', '(:)', '(:)', '(:)'], \
        a_bnds = ['1:object%dimv', 'ic:ic', '1:object%dimv', '1:object%dimv', \
                  '1:object%dimv', '1:object%dimv'], \
        b_bnds = ['1:f_md_obj%g_Att(Att_id)%dimv', 'ic:ic',
                  '1:f_md_obj%g_Att(Att_id)%dimv',
                  '1:f_md_obj%g_Att(Att_id)%dimv',
                  '1:f_md_obj%g_Att(Att_id)%dimv',
                  '1:f_md_obj%g_Att(Att_id)%dimv'], \
        c_bnds = ['1:f_md_obj%var_Att(Att_id,var_idx)%dimv', 'ic:ic',
                  '1:f_md_obj%var_Att(Att_id,var_idx)%dimv',
                  '1:f_md_obj%var_Att(Att_id,var_idx)%dimv',
                  '1:f_md_obj%var_Att(Att_id,var_idx)%dimv',
                  '1:f_md_obj%var_Att(Att_id,var_idx)%dimv'] \
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''manage_NetCDF_classic.F90.inc01'''
