!```````````````````````````````````````````````````````````````````
module univ_MPI_F90_intf_mod

!+ Module that contains routines that interface with the Fortran 90 binding of
!   a standard MPI library
!
!  If MPI_DEBUG (a preprocessor definition) is defined, copious debugging
!   information will be produced by these routines.

use univ_kind_defs_mod, only: i1

implicit none

type MPI_buffer_packed_obj
   integer(i1), pointer, dimension(:) :: data => NULL()
   integer :: pos,  &    ! current position (indexing begins at 0) in buffer
              nB      ! total allocated size of memory buffer (bytes)
end type MPI_buffer_packed_obj

interface mpiq_pack_buffer
   module procedure mpiq_pack_buffer_i1_0d
   module procedure mpiq_pack_buffer_i2_0d
   module procedure mpiq_pack_buffer_i4_0d
   module procedure mpiq_pack_buffer_i8_0d
   module procedure mpiq_pack_buffer_f4_0d
   module procedure mpiq_pack_buffer_f8_0d
   module procedure mpiq_pack_buffer_c_0d
   module procedure mpiq_pack_buffer_b_0d
   module procedure mpiq_pack_buffer_i1_1d
   module procedure mpiq_pack_buffer_i2_1d
   module procedure mpiq_pack_buffer_i4_1d
   module procedure mpiq_pack_buffer_i8_1d
   module procedure mpiq_pack_buffer_f4_1d
   module procedure mpiq_pack_buffer_f8_1d
   module procedure mpiq_pack_buffer_c_1d
   module procedure mpiq_pack_buffer_b_1d
   module procedure mpiq_pack_buffer_i1_2d
   module procedure mpiq_pack_buffer_i2_2d
   module procedure mpiq_pack_buffer_i4_2d
   module procedure mpiq_pack_buffer_i8_2d
   module procedure mpiq_pack_buffer_f4_2d
   module procedure mpiq_pack_buffer_f8_2d
   module procedure mpiq_pack_buffer_c_2d
   module procedure mpiq_pack_buffer_b_2d
   module procedure mpiq_pack_buffer_i1_3d
   module procedure mpiq_pack_buffer_i2_3d
   module procedure mpiq_pack_buffer_i4_3d
   module procedure mpiq_pack_buffer_i8_3d
   module procedure mpiq_pack_buffer_f4_3d
   module procedure mpiq_pack_buffer_f8_3d
   module procedure mpiq_pack_buffer_c_3d
   module procedure mpiq_pack_buffer_b_3d
   module procedure mpiq_pack_buffer_i1_4d
   module procedure mpiq_pack_buffer_i2_4d
   module procedure mpiq_pack_buffer_i4_4d
   module procedure mpiq_pack_buffer_i8_4d
   module procedure mpiq_pack_buffer_f4_4d
   module procedure mpiq_pack_buffer_f8_4d
   module procedure mpiq_pack_buffer_c_4d
   module procedure mpiq_pack_buffer_b_4d
end interface

interface mpiq_unpack_buffer
   module procedure mpiq_unpack_buffer_i1_0d
   module procedure mpiq_unpack_buffer_i2_0d
   module procedure mpiq_unpack_buffer_i4_0d
   module procedure mpiq_unpack_buffer_i8_0d
   module procedure mpiq_unpack_buffer_f4_0d
   module procedure mpiq_unpack_buffer_f8_0d
   module procedure mpiq_unpack_buffer_c_0d
   module procedure mpiq_unpack_buffer_b_0d
   module procedure mpiq_unpack_buffer_i1_1d
   module procedure mpiq_unpack_buffer_i2_1d
   module procedure mpiq_unpack_buffer_i4_1d
   module procedure mpiq_unpack_buffer_i8_1d
   module procedure mpiq_unpack_buffer_f4_1d
   module procedure mpiq_unpack_buffer_f8_1d
   module procedure mpiq_unpack_buffer_c_1d
   module procedure mpiq_unpack_buffer_b_1d
   module procedure mpiq_unpack_buffer_i1_2d
   module procedure mpiq_unpack_buffer_i2_2d
   module procedure mpiq_unpack_buffer_i4_2d
   module procedure mpiq_unpack_buffer_i8_2d
   module procedure mpiq_unpack_buffer_f4_2d
   module procedure mpiq_unpack_buffer_f8_2d
   module procedure mpiq_unpack_buffer_c_2d
   module procedure mpiq_unpack_buffer_b_2d
   module procedure mpiq_unpack_buffer_i1_3d
   module procedure mpiq_unpack_buffer_i2_3d
   module procedure mpiq_unpack_buffer_i4_3d
   module procedure mpiq_unpack_buffer_i8_3d
   module procedure mpiq_unpack_buffer_f4_3d
   module procedure mpiq_unpack_buffer_f8_3d
   module procedure mpiq_unpack_buffer_c_3d
   module procedure mpiq_unpack_buffer_b_3d
   module procedure mpiq_unpack_buffer_i1_4d
   module procedure mpiq_unpack_buffer_i2_4d
   module procedure mpiq_unpack_buffer_i4_4d
   module procedure mpiq_unpack_buffer_i8_4d
   module procedure mpiq_unpack_buffer_f4_4d
   module procedure mpiq_unpack_buffer_f8_4d
   module procedure mpiq_unpack_buffer_c_4d
   module procedure mpiq_unpack_buffer_b_4d
end interface

interface mpiq_transmit_plain
   module procedure mpiq_transmit_plain_i1_0d
   module procedure mpiq_transmit_plain_i2_0d
   module procedure mpiq_transmit_plain_i4_0d
   module procedure mpiq_transmit_plain_i8_0d
   module procedure mpiq_transmit_plain_f4_0d
   module procedure mpiq_transmit_plain_f8_0d
   module procedure mpiq_transmit_plain_c_0d
   module procedure mpiq_transmit_plain_b_0d
   module procedure mpiq_transmit_plain_i1_1d
   module procedure mpiq_transmit_plain_i2_1d
   module procedure mpiq_transmit_plain_i4_1d
   module procedure mpiq_transmit_plain_i8_1d
   module procedure mpiq_transmit_plain_f4_1d
   module procedure mpiq_transmit_plain_f8_1d
   module procedure mpiq_transmit_plain_c_1d
   module procedure mpiq_transmit_plain_b_1d
end interface

interface mpiq_receive_plain
   module procedure mpiq_receive_plain_i1_0d
   module procedure mpiq_receive_plain_i2_0d
   module procedure mpiq_receive_plain_i4_0d
   module procedure mpiq_receive_plain_i8_0d
   module procedure mpiq_receive_plain_f4_0d
   module procedure mpiq_receive_plain_f8_0d
   module procedure mpiq_receive_plain_c_0d
   module procedure mpiq_receive_plain_b_0d
   module procedure mpiq_receive_plain_i1_1d
   module procedure mpiq_receive_plain_i2_1d
   module procedure mpiq_receive_plain_i4_1d
   module procedure mpiq_receive_plain_i8_1d
   module procedure mpiq_receive_plain_f4_1d
   module procedure mpiq_receive_plain_f8_1d
   module procedure mpiq_receive_plain_c_1d
   module procedure mpiq_receive_plain_b_1d
end interface


contains


#include "univ_MPI_F90_intf.F90.inc01"

#include "univ_MPI_F90_intf.F90.inc02"

#include "univ_MPI_F90_intf.F90.inc03"


!```````````````````````````````````````````````````````````````````
subroutine mpiq_wait(msg_request)

!+ Waits for a MPI transmit (send) or receive operation to finish

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_SUCCESS
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_STATUS_SIZE, MPI_Wait

implicit none

!=== Argument declarations:

integer, intent(IN) :: msg_request

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_wait'

integer :: ierr, ireq

integer, dimension(MPI_STATUS_SIZE) :: msg_status

!=== Executable statements:

ireq = msg_request    ! Prevent 'msg_request' from being modified
call MPI_Wait(ireq, msg_status, ierr)

#if defined(MPI_DEBUG)
 write (*,*) procedure_name, ireq, msg_request; flush 6
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Wait failed.')
#endif  /* end:MPI_DEBUG */

end subroutine mpiq_wait


!```````````````````````````````````````````````````````````````````
subroutine mpiq_waitall(array_of_msg_requests, n_msg_requests)

!+ Waits for all of the specified MPI transmit (send) or receive operations to
!   finish

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_SUCCESS
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_STATUS_SIZE   !MPICH doesn't have this:  , MPI_Waitall

implicit none

!=== Argument declarations:

integer, intent(IN) :: n_msg_requests
integer, dimension(n_msg_requests), intent(IN) :: array_of_msg_requests

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_waitall'

integer :: ierr
integer, dimension(n_msg_requests) :: ireqs
integer, dimension(MPI_STATUS_SIZE, n_msg_requests) :: array_of_msg_statuses

!=== Executable statements:

ireqs = array_of_msg_requests    ! Prevent these from being modified
call MPI_Waitall(n_msg_requests, ireqs, array_of_msg_statuses, ierr)

#if defined(MPI_DEBUG)
 write (*,*) procedure_name, n_msg_requests, ireqs, array_of_msg_requests; flush 6
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Waitall failed.')
#endif  /* end:MPI_DEBUG */

end subroutine mpiq_waitall


!```````````````````````````````````````````````````````````````````
subroutine mpiq_wait_for_all(caller_rank, barrier_ID)

!+ Pauses execution of caller until all group members have called this routine

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_SUCCESS
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_COMM_WORLD, MPI_Barrier

implicit none

!=== Argument declarations:

integer, intent(IN) :: caller_rank, barrier_ID

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_wait_for_all'

integer :: ierr

!=== Executable statements:

write (*, '(a,i4,a,i9,a,/)') 'Waiting (this is PE #', caller_rank,  &
     ', barrier_ID ', barrier_ID, ') ...'; flush 6
  
call MPI_Barrier(MPI_COMM_WORLD, ierr)

#if defined(MPI_DEBUG)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Barrier failed.')
#endif  /* end:MPI_DEBUG */

end subroutine mpiq_wait_for_all


!```````````````````````````````````````````````````````````````````
subroutine mpiq_transmit_packed(tx_buffer_packed, destination_rank, msg_tag)

!+ Transmits (sends) a message (MPI_PACKED data in communication buffer) to a
!   specified PE
!
!  NOTE: A blocking MPI transmit procedure is used, therefore the memory buffer
!        specified may be safely reused/modified after this routine returns to
!        to its calling process

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_TAG_UB, MPI_SUCCESS, MPI_ADDRESS_KIND
#if !defined(BROKEN_MPI_MOD)
 use mpi, only: MPI_Comm_get_attr
#endif  /* end:not:BROKEN_MPI_MOD */
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_COMM_WORLD, MPI_PACKED  !not_in_mod MPI_Send

implicit none

!=== Argument declarations:

integer, intent(IN) :: destination_rank
integer, optional, intent(IN) :: msg_tag

type(MPI_buffer_packed_obj), intent(INOUT) :: tx_buffer_packed

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_transmit_packed'

integer :: ierr, tag
#if defined(MPI_DEBUG)
 integer(MPI_ADDRESS_KIND) :: this_tag_ub
 logical :: attr_was_extracted
#endif  /* end:MPI_DEBUG */

!=== Executable statements:

if (present(msg_tag)) then
   tag = msg_tag
else
   tag = 0
end if

#if defined(MPI_DEBUG)
 if (tag < 0) call exception(procedure_name, FATAL,  &
      'message tag is negative.')

 call MPI_Comm_get_attr(MPI_COMM_WORLD, MPI_TAG_UB, this_tag_ub,  &
      attr_was_extracted, ierr)
 if (attr_was_extracted) then
    if (tag > this_tag_ub) call exception(procedure_name, FATAL,  &
         'message tag is too large.')
 end if

 write (*,*) procedure_name, tx_buffer_packed%nB,  &
      tx_buffer_packed%pos, destination_rank, tag; flush 6

 call MPI_Send(tx_buffer_packed%data, tx_buffer_packed%pos, MPI_PACKED,  &
      destination_rank, tag, MPI_COMM_WORLD, ierr)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Send failed.')

#else  /* not:MPI_DEBUG */

 call MPI_Send(tx_buffer_packed%data, tx_buffer_packed%pos, MPI_PACKED,  &
      destination_rank, tag, MPI_COMM_WORLD, ierr)

#endif  /* end:MPI_DEBUG */

end subroutine mpiq_transmit_packed


!```````````````````````````````````````````````````````````````````
subroutine mpiq_receive_packed(rx_buffer_packed, source_rank, msg_tag)

!+ Places the contents of a received MPI message (composed of MPI_PACKED data)
!   into a receive buffer
!
!  NOTE: A blocking MPI receive procedure is used, therefore the memory buffer
!        specified may be safely reused/modified after this routine returns to
!        to its calling process

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_TAG_UB, MPI_SUCCESS, MPI_Get_count, MPI_ADDRESS_KIND
#if !defined(BROKEN_MPI_MOD)
 use mpi, only: MPI_Comm_get_attr
#endif  /* end:not:BROKEN_MPI_MOD */
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_STATUS_SIZE, MPI_ANY_SOURCE, MPI_COMM_WORLD, MPI_PACKED
     !not_in_mod MPI_Recv

implicit none

!=== Argument declarations:

integer, intent(IN) :: source_rank
integer, optional, intent(IN) :: msg_tag

type(MPI_buffer_packed_obj), intent(INOUT) :: rx_buffer_packed

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_receive_packed'

integer :: ierr, tag, rank_of_source

#if defined(MPI_DEBUG)
 integer :: n_msg_words
 integer(MPI_ADDRESS_KIND) :: this_tag_ub
 logical :: attr_was_extracted
#endif  /* end:MPI_DEBUG */

integer, dimension(MPI_STATUS_SIZE) :: msg_status

!=== Executable statements:

if (present(msg_tag)) then
   tag = msg_tag
else
   tag = 0
end if

if (source_rank >= 0) then
   rank_of_source = source_rank
else
   rank_of_source = MPI_ANY_SOURCE
end if

#if defined(MPI_DEBUG)
 if (tag < 0) call exception(procedure_name, FATAL,  &
      'message tag is negative.')

 call MPI_Comm_get_attr(MPI_COMM_WORLD, MPI_TAG_UB, this_tag_ub,  &
      attr_was_extracted, ierr)
 if (attr_was_extracted) then
    if (tag > this_tag_ub) call exception(procedure_name, FATAL,  &
         'message tag is too large.')
 end if

 write (*,*) procedure_name, 'A', rx_buffer_packed%nB,  &
      rx_buffer_packed%pos, source_rank, tag; flush 6

 call MPI_Recv(rx_buffer_packed%data, rx_buffer_packed%nB, MPI_PACKED,  &
      rank_of_source, tag, MPI_COMM_WORLD, msg_status, ierr)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Recv failed.')

 call MPI_Get_count(msg_status, MPI_PACKED, n_msg_words, ierr)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Get_count failed.')

 write (*,*) procedure_name, 'B', n_msg_words, tag; flush 6

#else  /* not:MPI_DEBUG */

 call MPI_Recv(rx_buffer_packed%data, rx_buffer_packed%nB, MPI_PACKED,  &
      rank_of_source, tag, MPI_COMM_WORLD, msg_status, ierr)

#endif  /* end:MPI_DEBUG */

end subroutine mpiq_receive_packed


!```````````````````````````````````````````````````````````````````
subroutine mpiq_check_buffer_packed(buffer_packed, new_buffer_packed_nB)

!+ Checks/allocates/resets/deallocates a buffer object for use with the
!   MPI_PACKED routines of this MPI interface module
!
!  If 'new_buffer_packed_nB' = 0, deallocate and/or nullify pointer storage in
!   the 'buffer_packed' object. 

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation

implicit none

!=== Argument declarations:

integer, intent(IN) :: new_buffer_packed_nB

type(MPI_buffer_packed_obj), intent(INOUT) :: buffer_packed

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_check_buffer_packed'

#if defined(MPI_DEBUG)
 integer :: ierr
#endif  /* end:MPI_DEBUG */

logical :: obj_pointer_is_assoc

!=== Executable statements:

obj_pointer_is_assoc = associated(buffer_packed%data)

if ( (.not.obj_pointer_is_assoc) .or.  &
     (new_buffer_packed_nB > buffer_packed%nB) .or.  &
     (new_buffer_packed_nB == 0) ) then

#if defined(MPI_DEBUG)
    if (obj_pointer_is_assoc) then
       deallocate(buffer_packed%data, stat=ierr)
       if (ierr /= 0) call illegal_alloc_stop(for_deallocation,  &
            procedure_name, 'A')
    end if

    write (*,'(a,1x,i9)') procedure_name, new_buffer_packed_nB; flush 6
    if (new_buffer_packed_nB > 0) then
       allocate(buffer_packed%data(0:new_buffer_packed_nB-1), stat=ierr)
       if (ierr /= 0) call illegal_alloc_stop(for_allocation,  &
            procedure_name, 'B')
    else
       buffer_packed%data => NULL()
    end if
#else  /* not:MPI_DEBUG */
    if (obj_pointer_is_assoc) deallocate(buffer_packed%data)
    if (new_buffer_packed_nB > 0) then
       allocate(buffer_packed%data(0:new_buffer_packed_nB-1))
    else
       buffer_packed%data => NULL()
    end if
#endif  /* end:MPI_DEBUG */

   buffer_packed%nB = new_buffer_packed_nB
end if

buffer_packed%pos = 0

end subroutine mpiq_check_buffer_packed


!```````````````````````````````````````````````````````````````````
subroutine mpiq_transmit_nonb_packed(tx_buffer_packed, destination_rank,  &
     msg_request, msg_tag)

!+ Identifies (for the MPI subsystem) a memory buffer that contains an outgoing
!   MPI message (non-blocking); initiates a transmit operation
!
!  NOTE: The memory buffer specified should *NOT* be modified in any way until
!        a call to MPI_Wait or MPI_Test indicates that the non-blocking transmit
!        has completed.

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_TAG_UB, MPI_SUCCESS, MPI_ADDRESS_KIND
#if !defined(BROKEN_MPI_MOD)
 use mpi, only: MPI_Comm_get_attr
#endif  /* end:not:BROKEN_MPI_MOD */
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_COMM_WORLD, MPI_PACKED  !not_in_mod MPI_Isend

implicit none

!=== Argument declarations:

integer, intent(IN) :: destination_rank
integer, optional, intent(IN) :: msg_tag

type(MPI_buffer_packed_obj), intent(INOUT) :: tx_buffer_packed

integer, intent(OUT) :: msg_request

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_transmit_nonb_packed'

integer :: ierr, tag
#if defined(MPI_DEBUG)
 integer(MPI_ADDRESS_KIND) :: this_tag_ub
 logical :: attr_was_extracted
#endif  /* end:MPI_DEBUG */

!=== Executable statements:

if (present(msg_tag)) then
   tag = msg_tag
else
   tag = 0
end if

#if defined(MPI_DEBUG)
 if (tag < 0) call exception(procedure_name, FATAL,  &
      'message tag is negative.')

 call MPI_Comm_get_attr(MPI_COMM_WORLD, MPI_TAG_UB, this_tag_ub,  &
      attr_was_extracted, ierr)
 if (attr_was_extracted) then
    if (tag < 0 .or. tag > this_tag_ub) call exception(procedure_name, FATAL,  &
         'message tag is too large.')
 end if

 write (*,*) procedure_name, 'A', tx_buffer_packed%nB,  &
      tx_buffer_packed%pos, destination_rank, tag; flush 6

 call MPI_Isend(tx_buffer_packed%data, tx_buffer_packed%pos, MPI_PACKED,  &
      destination_rank, tag, MPI_COMM_WORLD, msg_request, ierr)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Isend failed.')

 write (*,*) procedure_name, 'B', msg_request; flush 6

#else  /* not:MPI_DEBUG */

 call MPI_Isend(tx_buffer_packed%data, tx_buffer_packed%pos, MPI_PACKED,  &
      destination_rank, tag, MPI_COMM_WORLD, msg_request, ierr)

#endif  /* end:MPI_DEBUG */

end subroutine mpiq_transmit_nonb_packed


!```````````````````````````````````````````````````````````````````
subroutine mpiq_receive_nonb_packed(rx_buffer_packed, source_rank,  &
     msg_request, rxb_start_pos, msg_tag)

!+ Identifies (for the MPI subsystem) a memory buffer to receive an incoming
!   MPI message (non-blocking); initiates a receive
!   operation
!
!  NOTE: The memory buffer specified should *NOT* be modified in any way until
!        a call to MPI_Wait or MPI_Test indicates that the non-blocking receive
!        has completed.

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(MPI_DEBUG)
 use univ_exception_mod, only: exception, FATAL
 use mpi, only: MPI_TAG_UB, MPI_SUCCESS, MPI_ADDRESS_KIND
#if !defined(BROKEN_MPI_MOD)
 use mpi, only: MPI_Comm_get_attr
#endif  /* end:not:BROKEN_MPI_MOD */
#endif  /* end:MPI_DEBUG */
use mpi, only: MPI_COMM_WORLD, MPI_PACKED, MPI_ANY_SOURCE  !not_in_mod MPI_Irecv

implicit none

!=== Argument declarations:

integer, intent(IN) :: source_rank, rxb_start_pos
integer, optional, intent(IN) :: msg_tag

type(MPI_buffer_packed_obj), intent(INOUT) :: rx_buffer_packed

integer, intent(OUT) :: msg_request

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mpiq_receive_nonb_packed'

integer :: ierr, tag, rank_of_source
#if defined(MPI_DEBUG)
 integer(MPI_ADDRESS_KIND) :: this_tag_ub
 logical :: attr_was_extracted
#endif  /* end:MPI_DEBUG */

!=== Executable statements:

if (present(msg_tag)) then
   tag = msg_tag
else
   tag = 0
end if

if (source_rank >= 0) then
   rank_of_source = source_rank
else
   rank_of_source = MPI_ANY_SOURCE
end if

#if defined(MPI_DEBUG)
 if (tag < 0) call exception(procedure_name, FATAL,  &
      'message tag is negative.')

 call MPI_Comm_get_attr(MPI_COMM_WORLD, MPI_TAG_UB, this_tag_ub,  &
      attr_was_extracted, ierr)
 if (attr_was_extracted) then
    if (tag > this_tag_ub) call exception(procedure_name, FATAL,  &
         'message tag is too large.')
 end if

 write (*,*) procedure_name, 'A', rx_buffer_packed%nB, source_rank,  &
      tag, rxb_start_pos; flush 6

 call MPI_Irecv(rx_buffer_packed%data(rxb_start_pos:),  &
      rx_buffer_packed%nB-rxb_start_pos, MPI_PACKED, rank_of_source, tag,  &
      MPI_COMM_WORLD, msg_request, ierr)
 if (ierr /= MPI_SUCCESS) call exception(procedure_name, FATAL,  &
      'MPI_Irecv failed.')

 write (*,*) procedure_name, 'B', msg_request; flush 6

#else  /* not:MPI_DEBUG */

 call MPI_Irecv(rx_buffer_packed%data(rxb_start_pos:),  &
      rx_buffer_packed%nB-rxb_start_pos, MPI_PACKED, rank_of_source, tag,  &
      MPI_COMM_WORLD, msg_request, ierr)

#endif  /* end:MPI_DEBUG */

end subroutine mpiq_receive_nonb_packed


end module univ_MPI_F90_intf_mod
