"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict(
                    d = 'dimension',
                    n1 = 'n_words_1',
                    line_tx = '     destination_rank, msg_tag)',
                    line_rx = '     source_rank, msg_tag)'
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict(
         kind_idx_n = ['i1', 'i2', 'i4', 'i8', 'f4', 'f8', 'c', 'b'],
         kind_idx_use = ['use univ_kind_defs_mod, only: i1',
                         'use univ_kind_defs_mod, only: i2',
                         'use univ_kind_defs_mod, only: i4',
                         'use univ_kind_defs_mod, only: i8',
                         'use univ_kind_defs_mod, only: f4',
                         'use univ_kind_defs_mod, only: f8', ' ',
                         'use univ_kind_defs_mod, only: bn'],
         kind_idx_dec = ['i1', 'i2', 'i4', 'i8', 'f4', 'f8', 'len=*', 'bn'],
         kind_desc = ['1-byte integer', '2-byte integer', '4-byte integer',
                      '8-byte integer', '4-byte floating point',
                      '8-byte floating point', 'character', 'logical'],
         kind_type = ['integer', 'integer', 'integer', 'integer', 'real',
                      'real', 'character', 'logical'],
         mpi_dtype = ['MPI_INTEGER1', 'MPI_INTEGER2', 'MPI_INTEGER4',
                      'MPI_INTEGER8', 'MPI_REAL4', 'MPI_REAL8',
                      'MPI_CHARACTER', 'MPI_LOGICAL']
        ), \
    dict(
         dim_n = ['0d', '1d'],
         dim_line1_tx = ['${line_tx}', '     ${n1},  &'],
         dim_line2_tx = [' ', '${line_tx}'],
         dim_line1_rx = ['${line_rx}', '     ${n1},  &'],
         dim_line2_rx = [' ', '${line_rx}'],
         dim_args1 = [' ', ', n_words_1'],
         dim_desc = ['scalar', '1-dimensional'],
         dim_dec = [' ', ' ${d}(${n1}), '],
         dim_word_cnt = ['1', '${n1}']
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''univ_MPI_F90_intf.F90.inc01'''
