"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict(
                    d = 'dimension',
                    n1 = 'n_words_1',
                    n2 = 'n_words_2',
                    n3 = 'n_words_3',
                    n4 = 'n_words_4',
                    idec = 'integer, intent(IN) :: '
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict(
         kind_idx_n = ['i1', 'i2', 'i4', 'i8', 'f4', 'f8', 'b'],
         kind_idx_use = ['i1', 'i2', 'i4', 'i8', 'f4', 'f8', 'bn'],
         kind_idx_use_B = ['i1_B', 'i2_B', 'i4_B', 'i8_B', 'f4_B', 'f8_B',
                           'bn_B'],
         kind_idx_dec = ['i1', 'i2', 'i4', 'i8', 'f4', 'f8', 'bn'],
         kind_desc = ['1-byte integer', '2-byte integer', '4-byte integer',
                      '8-byte integer', '4-byte floating point',
                      '8-byte floating point', 'logical'],
         kind_type = ['integer', 'integer', 'integer', 'integer', 'real',
                      'real', 'logical'],
         mpi_dtype = ['MPI_INTEGER1', 'MPI_INTEGER2', 'MPI_INTEGER4',
                      'MPI_INTEGER8', 'MPI_REAL4', 'MPI_REAL8', 'MPI_LOGICAL']
        ), \
    dict(
         dim_n = ['0d', '1d', '2d', '3d', '4d'],
         dim_args = [' ', '${n1}, ', '${n1}, ${n2}, ', '${n1}, ${n2}, ${n3}, ',
                     '${n1}, ${n2}, ${n3}, ${n4}, '],
         dim_args1 = [' ', '${idec}${n1}', '${idec}${n1}, ${n2}',
                      '${idec}${n1}, ${n2}, ${n3}',
                      '${idec}${n1}, ${n2}, ${n3}, ${n4}'],
         dim_desc = ['scalar', '1-dimensional array', '2-dimensional array',
                     '3-dimensional array', '4-dimensional array'],
         dim_dec = [' ', '${d}(${n1}), ', '${d}(${n1}, ${n2}), ',
                      '${d}(${n1}, ${n2}, ${n3}), ',
                      '${d}(${n1}, ${n2}, ${n3}, ${n4}), '],
         dim_mult = ['1', '${n1}', '${n1}*${n2}',
                      '${n1}*${n2}*${n3}',
                      '${n1}*${n2}*${n3}*${n4}']
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''univ_MPI_F90_intf.F90.inc02'''
