"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict(
                    d = 'dimension',
                    n0 = 'n_words_0',
                    n1 = 'n_words_1',
                    n2 = 'n_words_2',
                    n3 = 'n_words_3',
                    n4 = 'n_words_4',
                    ed = 'end do',
                    idec = 'integer, intent(IN) :: '
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict(
         dim_n = ['0d', '1d', '2d', '3d', '4d'],
         dim_args = ['${n0}, ', '${n0}, ${n1}, ', '${n0}, ${n1}, ${n2}, ',
                     '${n0}, ${n1}, ${n2}, ${n3}, ',
                     '${n0}, ${n1}, ${n2}, ${n3}, ${n4}, '],
         dim_args1 = ['${idec}${n0}', '${idec}${n0}, ${n1}',
                      '${idec}${n0}, ${n1}, ${n2}',
                      '${idec}${n0}, ${n1}, ${n2}, ${n3}',
                      '${idec}${n0}, ${n1}, ${n2}, ${n3}, ${n4}'],
         dim_desc = ['string', '1-dimensional string array',
                     '2-dimensional string array', '3-dimensional string array',
                     '4-dimensional string array'],
         dim_dec = [' ', '${d}(${n1}), ', '${d}(${n1}, ${n2}), ',
                      '${d}(${n1}, ${n2}, ${n3}), ',
                      '${d}(${n1}, ${n2}, ${n3}, ${n4}), '],
         dim_mult = ['${n0}', '${n0}*${n1}', '${n0}*${n1}*${n2}',
                      '${n0}*${n1}*${n2}*${n3}',
                      '${n0}*${n1}*${n2}*${n3}*${n4}'],
         dim_aidec = [' ', ', i1', ', i1, i2', ', i1, i2, i3',
                      ', i1, i2, i3, i4'],
         dim_aidx = [' ', '(i1)', '(i1,i2)', '(i1,i2,i3)', '(i1,i2,i3,i4)'],
         dim_loopb1 = [' ', 'do i1=1,${n1}', 'do i1=1,${n1}', 'do i1=1,${n1}',
                       'do i1=1,${n1}'],
         dim_loopb2 = [' ', ' ', 'do i2=1,${n2}', 'do i2=1,${n2}',
                       'do i2=1,${n2}'],
         dim_loopb3 = [' ', ' ', ' ', 'do i3=1,${n3}', 'do i3=1,${n3}'],
         dim_loopb4 = [' ', ' ', ' ', ' ', 'do i4=1,${n4}'],
         dim_loope1 = [' ', '${ed}', '${ed}', '${ed}', '${ed}'],
         dim_loope2 = [' ', ' ', '${ed}', '${ed}', '${ed}'],
         dim_loope3 = [' ', ' ', ' ', '${ed}', '${ed}'],
         dim_loope4 = [' ', ' ', ' ', ' ', '${ed}']
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''univ_MPI_F90_intf.F90.inc03'''
