!```````````````````````````````````````````````````````````````````
module univ_math_constants_mod

!+ Module containing "universal" mathematical constants

use univ_kind_defs_mod, only: f8, f4

implicit none

!--- "Higher" precision definitions (see next section for "normal" precision):

real(f8), parameter ::  &
     pi_f8 = 3.1415926535897932_f8,  & ! Mathematical Pi [none]
     twopi_f8 = pi_f8*2._f8,  &  ! Circumference of a unit circle [rad]
     fourpi_f8 = pi_f8*4._f8,  & !
     ONE_f8 = 1._f8,  & ! One
     ZERO_f8 = 0._f8,  &  ! Zero
     sqrt2_f8 = 1.4142135623731_f8,  & ! Square root of 2
     sqrt3_f8 = 1.7320508075689_f8,  &  ! Square root of 3
     ONE_HALF_f8 = ONE_f8/2._f8,  & ! 1/2
     ONE_THIRD_f8 = ONE_f8/3._f8,  &  ! 1/3
     ONE_FOURTH_f8 = ONE_f8/4._f8,  & ! 1/4
     ONE_FIFTH_f8 = ONE_f8/5._f8,  &  ! 1/5
     ONE_SIXTH_f8 = ONE_f8/6._f8,  & ! 1/6
     ONE_SEVENTH_f8 = ONE_f8/7._f8,  &  ! 1/7
     ONE_EIGHTH_f8 = ONE_f8/8._f8,  & ! 1/8
     ONE_NINTH_f8 = ONE_f8/9._f8,  &  ! 1/9
     TWO_THIRDS_f8 = 2._f8/3._f8,  & ! 2/3
     FOUR_THIRDS_f8 = 4._f8/3._f8      ! 4/3


!--- "Normal" precision definitions:
!
!    Note: explicitly calculate (with *_f4 constants) any values that were
!    calculated in the "higher" precision section -- for numerical consistency

real(f4), parameter ::  &
     pi_f4 = real(pi_f8, kind=f4),  &
     twopi_f4 = pi_f4*2._f4,  &
     fourpi_f4 = pi_f4*4._f4,  &
     ONE_f4 = 1._f4,  & 
     ZERO_f4 = 0._f4,  &
     sqrt2_f4 = real(sqrt2_f8, kind=f4),  &
     sqrt3_f4 = real(sqrt3_f8, kind=f4),  & 
     ONE_HALF_f4 = ONE_f4/2._f4,  &
     ONE_THIRD_f4 = ONE_f4/3._f4,  &
     ONE_FOURTH_f4 = ONE_f4/4._f4,  &
     ONE_FIFTH_f4 = ONE_f4/5._f4,  &
     ONE_SIXTH_f4 = ONE_f4/6._f4,  &
     ONE_SEVENTH_f4 = ONE_f4/7._f4,  &
     ONE_EIGHTH_f4 = ONE_f4/8._f4,  &
     ONE_NINTH_f4 = ONE_f4/9._f4,  &
     TWO_THIRDS_f4 = 2._f4/3._f4,  &
     FOUR_THIRDS_f4 = 4._f4/3._f4           

end module univ_math_constants_mod
