!```````````````````````````````````````````````````````````````````
module univ_maxlen_string_mod

!+ Module that provides (maximum) reasonable character string lengths for
!   common usage.
!
!  These definitions exist in a separate module/file so that fundamental
!   modules/procedures can utilize them without fear of recursive dependencies.

implicit none

! Maximum character length of procedure name strings [Fortran "char units"].
!  Fortran 90+ standard dictates 31 character limit:
integer, parameter :: MAX_PROC_NAME_LEN = 31

! Maximum character length of file name strings [Fortran "char units"]:
integer, parameter :: MAX_FILE_NAME_LEN = 256,  &   ! Without path
                      MAX_FILE_PATH_LEN = 4096    ! With path

! Maximum character length of a generic message string [Fortran "char units"]:
integer, parameter :: MAX_MESSAGE_LEN = 4096

end module univ_maxlen_string_mod
