!```````````````````````````````````````````````````````````````````
module univ_IO_mem_status_mod

!+ Routines related to status checks for I/O, memory allocation, pointers, etc.

implicit none

logical, parameter :: for_allocation = .TRUE.,  &
                              for_deallocation = .FALSE.


contains


!```````````````````````````````````````````````````````````````````
subroutine illegal_alloc_stop(alloc_sense, calling_procedure_name, ID_string)

!+ Displays an appropriate error message and generates a FATAL exception.

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN
!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

logical, intent(IN) :: alloc_sense
character(len=*), intent(IN) :: calling_procedure_name, ID_string

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'illegal_alloc_stop'

character(len=MAX_MESSAGE_LEN) :: output_buffer

!=== Executable statements:

if (alloc_sense) then   ! Allocation being checked
   write (output_buffer,'(4a)') 'Memory allocation failed at marker `',  &
        trim(ID_string),  &
        '`, which may be due to a shortage of memory (RAM) ',  &
        'or the object may have already been allocated.'
else                   ! DEallocation being checked
   write (output_buffer,'(4a)') 'Memory DEallocation failed at marker `',  &
        trim(ID_string),  &
        '`, which may be due to a memory "leak" within this program ',  &
        'or the object may have not actually been allocated.'
end if

call exception(calling_procedure_name, FATAL, output_buffer)

end subroutine illegal_alloc_stop


end module univ_IO_mem_status_mod
