"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict( \
                   use_kd_mod = 'use univ_kind_defs_mod, only:' \
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict( \
        kind_idx_n = ['i1', 'i2', 'i4', 'f4', 'f8'], \
        kind_idx_use = ['i1', 'i2', 'i4', 'f4', 'f8'], \
        kind_idx_dec = ['i1', 'i2', 'i4', 'f4', 'f8'], \
        kind_desc = ['1-byte integer', '2-byte integer', '4-byte integer', \
                     '4-byte floating point', '8-byte floating point'], \
        kind_mod = ['${use_kd_mod}', '${use_kd_mod}', '${use_kd_mod}', \
                    '${use_kd_mod}', '${use_kd_mod}'], \
        kind_type = ['integer', 'integer', 'integer', 'real', 'real'], \
        c_int = ['!', '!', '!', ' ', ' '], \
        c_fp = [' ', ' ', ' ', '!', '!'] \
        ), \
    dict( \
        dim_n = ['0d', '1d'], \
        dim_desc = ['scalar', '1-dimensional'], \
        dim_dec = [' ', ', dimension(:)'], \
        minval = [' ', 'minval'], \
        maxval = [' ', 'maxval'], \
        eq_extra_line = [' ', \
           'value_error = value_error.or.(.not.(maxval(input_data) == eq))'], \
        ne_extra_line = [' ', \
           'value_error = value_error.or.(.not.(maxval(input_data) /= eq))'], \
        n_elems_cont = [' ', ', &'], \
        n_elems_msg = [' ', \
           "' array must have at least ', size(input_data), ' elements ;'"] \
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''univ_namelist_utils.f90.inc01'''
