"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict( \
                   use_kd_mod = 'use univ_kind_defs_mod, only:'
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict( \
        kind_idx_n = ['i1', 'c', 'i2', 'i4', 'i8', 'f4', 'f8', 'b'],
        kind_idx_use = ['i1', ' ', 'i2', 'i4', 'i8', 'f4', 'f8', 'bn'],
        kind_idx_dec = ['i1', 'len=*', 'i2', 'i4', 'i8', 'f4', 'f8', 'bn'],
        kind_idx_use_1 = [' ',
                          'use univ_maxlen_string_mod, only:  MAX_MESSAGE_LEN',
                          ' ', ' ', ' ', ' ', ' ', ' '],
        kind_idx_dec_1 = ['i1', 'len=MAX_MESSAGE_LEN', 'i2', 'i4', 'i8', 'f4',
                          'f8', 'bn'],
        kind_desc = ['1-byte integer', 'character', '2-byte integer',
                     '4-byte integer', '8-byte integer',
                     '4-byte floating point', '8-byte floating point',
                     'boolean'],
        kind_mod = ['${use_kd_mod}', ' ', '${use_kd_mod}', '${use_kd_mod}',
                    '${use_kd_mod}', '${use_kd_mod}', '${use_kd_mod}',
                    '${use_kd_mod}'],
        kind_type = ['integer', 'character', 'integer', 'integer', 'integer',
                     'real', 'real', 'logical']
        ), \
    dict( \
        dim_n = ['1d', '2d', '3d', '4d', '5d', '6d', '7d'],
        dim_desc = ['1-dimensional', '2-dimensional', '3-dimensional',
                    '4-dimensional', '5-dimensional', '6-dimensional',
                    '7-dimensional'],
        dim_alloc_1 = ['pd(1)', 'pd(1), pd(2)', 'pd(1), pd(2), pd(3)',
                       'pd(1), pd(2), pd(3), pd(4)',
                       'pd(1), pd(2), pd(3), pd(4), pd(5)',
                       'pd(1), pd(2), pd(3), pd(4), pd(5), pd(6)',
                       'pd(1), pd(2), pd(3), pd(4), pd(5), pd(6), pd(7)'],
        dim_alloc_2 = ['new_pd(1)', 'new_pd(1), new_pd(2)',
                       'new_pd(1), new_pd(2), new_pd(3)',
                       'new_pd(1), new_pd(2), new_pd(3), new_pd(4)',
        'new_pd(1), new_pd(2), new_pd(3), new_pd(4), new_pd(5)',
        'new_pd(1), new_pd(2), new_pd(3), new_pd(4), new_pd(5), new_pd(6)',
 'new_pd(1), new_pd(2), new_pd(3), new_pd(4), new_pd(5), new_pd(6), new_pd(7)'],
        dim_a_slice = ['1:pd(1)', '1:pd(1),1:pd(2)',
                       '1:pd(1),1:pd(2),1:pd(3)',
                       '1:pd(1),1:pd(2),1:pd(3),1:pd(4)',
                       '1:pd(1),1:pd(2),1:pd(3),1:pd(4),1:pd(5)',
                       '1:pd(1),1:pd(2),1:pd(3),1:pd(4),1:pd(5),1:pd(6)',
                  '1:pd(1),1:pd(2),1:pd(3),1:pd(4),1:pd(5),1:pd(6),1:pd(7)'],
        dim_rank_dec = ['(:)', '(:,:)', '(:,:,:)', '(:,:,:,:)', '(:,:,:,:,:)',
                        '(:,:,:,:,:,:)', '(:,:,:,:,:,:,:)']
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''univ_pointer_utils.f90.inc01'''
