!```````````````````````````````````````````````````````````````````
module univ_std_fileIO_mod

!+ Contains routines related to "standard" Fortran file input/output

implicit none

integer, parameter :: NEWLINE_NONE = 0, NEWLINE_BEG = 1, NEWLINE_END = 2,  &
     NEWLINE_BOTH = 3

interface determine_IO_reclen
   module procedure determine_IO_reclen_i1_0d, determine_IO_reclen_i1_1d,  &
        determine_IO_reclen_i1_2d, determine_IO_reclen_i1_3d,  &
        determine_IO_reclen_i1_4d, determine_IO_reclen_i1_5d,  &
        determine_IO_reclen_i1_6d, determine_IO_reclen_i1_7d
   module procedure determine_IO_reclen_c1_0d, determine_IO_reclen_c1_1d,  &
        determine_IO_reclen_c1_2d, determine_IO_reclen_c1_3d,  &
        determine_IO_reclen_c1_4d, determine_IO_reclen_c1_5d,  &
        determine_IO_reclen_c1_6d, determine_IO_reclen_c1_7d
   module procedure determine_IO_reclen_i2_0d, determine_IO_reclen_i2_1d,  &
        determine_IO_reclen_i2_2d, determine_IO_reclen_i2_3d,  &
        determine_IO_reclen_i2_4d, determine_IO_reclen_i2_5d,  &
        determine_IO_reclen_i2_6d, determine_IO_reclen_i2_7d
   module procedure determine_IO_reclen_i4_0d, determine_IO_reclen_i4_1d,  &
        determine_IO_reclen_i4_2d, determine_IO_reclen_i4_3d,  &
        determine_IO_reclen_i4_4d, determine_IO_reclen_i4_5d,  &
        determine_IO_reclen_i4_6d, determine_IO_reclen_i4_7d
   module procedure determine_IO_reclen_i8_0d, determine_IO_reclen_i8_1d,  &
        determine_IO_reclen_i8_2d, determine_IO_reclen_i8_3d,  &
        determine_IO_reclen_i8_4d, determine_IO_reclen_i8_5d,  &
        determine_IO_reclen_i8_6d, determine_IO_reclen_i8_7d
   module procedure determine_IO_reclen_f4_0d, determine_IO_reclen_f4_1d,  &
        determine_IO_reclen_f4_2d, determine_IO_reclen_f4_3d,  &
        determine_IO_reclen_f4_4d, determine_IO_reclen_f4_5d,  &
        determine_IO_reclen_f4_6d, determine_IO_reclen_f4_7d
   module procedure determine_IO_reclen_f8_0d, determine_IO_reclen_f8_1d,  &
        determine_IO_reclen_f8_2d, determine_IO_reclen_f8_3d,  &
        determine_IO_reclen_f8_4d, determine_IO_reclen_f8_5d,  &
        determine_IO_reclen_f8_6d, determine_IO_reclen_f8_7d
   module procedure determine_IO_reclen_b_0d, determine_IO_reclen_b_1d,  &
        determine_IO_reclen_b_2d, determine_IO_reclen_b_3d,  &
        determine_IO_reclen_b_4d, determine_IO_reclen_b_5d,  &
        determine_IO_reclen_b_6d, determine_IO_reclen_b_7d
end interface


contains


!```````````````````````````````````````````````````````````````````
function find_file_IOunit(start_IOunit) result(available_IOunit)

!+ Returns the (integer) ID of the first available file Fortran I/O unit
!
!  If the optional argument is present, the Fortran file I/O unit returned will
!   be >= 'start_IOunit'

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

integer, optional, intent(IN) :: start_IOunit

integer :: available_IOunit      ! Procedure result

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'find_file_IOunit'

integer, parameter :: max_file_IOunit = 99, min_file_IOunit = 20

integer :: iu, iu_beg
logical :: is_open, file_IOunit_found

!=== Executable statements:

if (present(start_IOunit)) then
   iu_beg = max(min_file_IOunit, start_IOunit)
else
   iu_beg = min_file_IOunit
end if

file_IOunit_found = .FALSE.
do iu=iu_beg,max_file_IOunit
   inquire(iu, opened=is_open)
   if (.NOT.is_open) then     ! This file_IOunit is currently "available"
      file_IOunit_found = .TRUE.
      exit   ! Exit loop
   end if
end do

if (.NOT.file_IOunit_found) then
   call exception(procedure_name, FATAL,  &
        'No available Fortran file I/O units were found.')
end if

available_IOunit = iu

end function find_file_IOunit


!```````````````````````````````````````````````````````````````````
subroutine inform_user(IOunit_flag, newline_flag, message)

!+ Routine that informs the program user by writing a given message to stdout
!   and/or an auxiliary file unit (which must already be opened for "formatted"
!   output), with or without extra surrounding newline(s).
!
!  IOunit_flag = 0  :  Write to stdout only
!  IOunit_flag < 0  :  Write only to file unit abs(IOunit_flag)
!  IOunit_flag > 0  :  Write to *both* stdout and file unit IOunit_flag
!
!  newline_flag = NEWLINE_NONE  :  add no extra newlines
!               = NEWLINE_BEG  :  add an extra newline before the message
!               = NEWLINE_END  :  add an extra newline after the message
!               = NEWLINE_BOTH  :  an extra newline before and after the message

use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN

implicit none

!=== Argument declarations:

integer, intent(IN) :: IOunit_flag, newline_flag
character(len=*), intent(IN) :: message

!== Local declarations:

integer :: ic, aux_IOunit
logical :: use_std, use_aux

!=== Executable statements:

! Determine some control parameters:
use_std = ( IOunit_flag >= 0 )
use_aux = ( IOunit_flag /= 0 )
if (use_aux) aux_IOunit = abs(IOunit_flag)

! Limit input character string to a "reasonable" length:
ic = min(MAX_MESSAGE_LEN, len_trim(message))

! Populate output buffer; set abort toggle:
select case (newline_flag)
case (NEWLINE_NONE)
   if (use_std) write (*, '(a)') message(1:ic)
   if (use_aux) write (aux_IOunit, '(a)') message(1:ic)
case (NEWLINE_BEG)
   if (use_std) write (*, '(/,a)') message(1:ic)
   if (use_aux) write (aux_IOunit, '(/,a)') message(1:ic)
case (NEWLINE_END)
   if (use_std) write (*, '(a,/)') message(1:ic)
   if (use_aux) write (aux_IOunit, '(a,/)') message(1:ic)
case (NEWLINE_BOTH)
   if (use_std) write (*, '(/,a,/)') message(1:ic)
   if (use_aux) write (aux_IOunit, '(/,a,/)') message(1:ic)
end select

end subroutine inform_user


!```````````````````````````````````````````````````````````````````
subroutine mkdir_p(path_to_create, op_was_successful)

!+ Checks/creates a given directory, along with any necessary parent directories

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: path_to_create
logical, optional, intent(OUT) :: op_was_successful

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mkdir_p'

character(len=MAX_FILE_PATH_LEN) :: f_path_tmp
integer :: path_len, err

! Explicit interface for C routine (to help the compiler catch errors):
interface
   subroutine univ_mkdir_p_f(e_arg1, e_arg2, e_arg3)
      integer, intent(INOUT) :: e_arg1, e_arg3
      character(len=*), intent(INOUT) :: e_arg2
   end subroutine univ_mkdir_p_f
end interface

!=== Executable statements:

path_len = min(len_trim(path_to_create), MAX_FILE_PATH_LEN)
write (*, '(/,4a)') '{', trim(procedure_name), '}: Checking/creating path: ',  &
     path_to_create(1:path_len)

! Create specified directory (tree):

! `univ_mkdir_p_f` is implemented as a C routine:
f_path_tmp = path_to_create   ! local copy of intent(IN) argument
call univ_mkdir_p_f(path_len, f_path_tmp, err)

if (present(op_was_successful)) op_was_successful = ( err == 0 )

end subroutine mkdir_p


include "univ_std_fileIO.f90.inc01"


end module univ_std_fileIO_mod
