!```````````````````````````````````````````````````````````````````
module univ_EC_proj_mod

!~ 11.11.2008 (last modified; TIM)

!+ Equidistant cyclindrical (EC) coordinate routines

implicit none

private

public :: P3C_xy_EC

contains


!```````````````````````````````````````````````````````````````````
subroutine P3C_xy_EC(xpq,ypq,zpq,EC_lon,EC_lat)

!~ 11.11.2008 (last modified; TIM)

!+ "Planetary 3D Cartesian" (P3C) coordinates to lon/lat equidistant
!   cyclindrical (EC) coordinates
!
!  This subroutine computes coordinates (EC_lon,EC_lat) of point q projected
!   onto an equidistant cyclindrical surface.  Input coordinates (xpq,ypq,zpq)
!   of q are defined in "Planetary 3D Cartesian" (P3C) space, where the origin
!   is the center of the planet, the z axis is the north pole, the x axis is
!   the equator and prime meridian, and the y axis is the equator and 90E.

use univ_kind_defs_mod, only: f4
use univ_convert_units_mod, only: rad_to_deg_f4

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: xpq, ypq, zpq

real(f4), intent(OUT) :: EC_lat, EC_lon    ! [deg]

!== Local declarations:

real(f4) :: rax

!=== Executable statements:

rax = sqrt(xpq**2+ypq**2)    ! Distance of q from planetary axis
EC_lat = atan2(zpq,rax)*rad_to_deg_f4
EC_lon = atan2(ypq,xpq)*rad_to_deg_f4

end subroutine P3C_xy_EC


end module univ_EC_proj_mod
