!```````````````````````````````````````````````````````````````````
module univ_histogram_mod

!+ Contains a framework for computing histograms

use univ_kind_defs_mod, only: f4, f8, i1, i2, i4, i8

implicit none

! Can be used to accumulate histogram frequency counts as in the below example:
!
!     call histogram_eqb_init(obj, ..., .TRUE.)
!     call histogram_eqb_populate(obj, data1)
!     call histogram_eqb_populate(obj, data2)
!              ...
!     call histogram_obj_recycle(obj)      {OPTIONAL}

include 'univ_histogram.f90.inc02'

type permutation_2D_obj
   integer, dimension(2) :: ib
end type permutation_2D_obj

interface histogram_obj_recycle
   module procedure histogram_1D_obj_recycle_f4, histogram_1D_obj_recycle_f8
   module procedure histogram_2D_obj_recycle_f4, histogram_2D_obj_recycle_f8
end interface

interface histogram_eqb_init
   module procedure histogram_eqb_1D_init_f4, histogram_eqb_1D_init_f8
   module procedure histogram_eqb_2D_init_f4, histogram_eqb_2D_init_f8  
end interface

interface histogram_eqb_populate
   module procedure histogram_eqb_1D_populate_f4, histogram_eqb_1D_populate_f8
   module procedure histogram_eqb_2D_populate_f4, histogram_eqb_2D_populate_f8
end interface

interface histogram_vrb_init
!%   module procedure histogram_vrb_1D_init_f4, histogram_vrb_1D_init_f8
   module procedure histogram_vrb_2D_init_f4, histogram_vrb_2D_init_f8  
end interface

interface histogram_vrb_populate
!%   module procedure histogram_vrb_1D_populate_f4, histogram_vrb_1D_populate_f8
   module procedure histogram_vrb_2D_populate_f4, histogram_vrb_2D_populate_f8
end interface


contains


include 'univ_histogram.f90.inc01'


end module univ_histogram_mod
